@@ -25,10 +25,7 @@
 #include "mesh.h"
 #include "reference.h"
 
-#include<sys/types.h>
-#include<sys/stat.h>
-#include<dirent.h>
-#include <dlfcn.h>
+#include "yafsystem.h"
 
 using namespace std;
 
@@ -38,28 +35,6 @@ using namespace std;
 
 __BEGIN_YAFRAY
 
-list<string> listDir(const string &dir)
-{
-	DIR *directorio;
-	struct dirent *entrada;
-	struct stat estado;
-	list<string> lista;
-
-	directorio=opendir(dir.c_str());
-	if(directorio==NULL) return lista;
-
-	entrada=readdir(directorio);
-	while(entrada!=NULL) 
-	{
-		string full=dir+"/"+entrada->d_name;
-		stat(full.c_str(),&estado);
-		if(S_ISREG(estado.st_mode))
-			lista.push_back(full);
-    entrada=readdir(directorio);
-  }
-  closedir(directorio);
-	return lista;
-}
 
 interfaceImpl_t::interfaceImpl_t(int ncpus,const string &pluginpath):M(1.0)
 {
@@ -72,7 +47,7 @@ interfaceImpl_t::~interfaceImpl_t()
 {
 #define MAPOF(type) map<string,type *>
 #define FREEMAP(type,name)\
-	for(MAPOF(type)::iterator i=name.begin();i!=name.end();++i) delete i->second
+	for(MAPOF(type)::iterator i=name.begin();i!=name.end();++i) delete i->second;name.clear()
 
 	FREEMAP(texture_t,texture_table);
 	FREEMAP(shader_t,shader_table);
@@ -83,9 +58,24 @@ interfaceImpl_t::~interfaceImpl_t()
 	FREEMAP(background_t,background_table);
 #undef FREEMAP
 #undef MAPOF
-	for(list<void *>::iterator i=pluginHandlers.begin();i!=pluginHandlers.end();
-			++i)
-		dlclose(*i);
+}
+
+void interfaceImpl_t::clear()
+{
+#define MAPOF(type) map<string,type *>
+#define FREEMAP(type,name)\
+	for(MAPOF(type)::iterator i=name.begin();i!=name.end();++i) delete i->second;name.clear()
+	FREEMAP(texture_t,texture_table);
+	FREEMAP(shader_t,shader_table);
+	FREEMAP(object3d_t,object_table);
+	FREEMAP(camera_t,camera_table);
+	FREEMAP(light_t,light_table);
+	FREEMAP(filter_t,filter_table);
+	FREEMAP(background_t,background_table);
+	cachedPathLight=false;
+	tstack.clear();
+#undef FREEMAP
+#undef MAPOF
 }
 
 void interfaceImpl_t::loadPlugins(const string &path)
@@ -95,25 +85,13 @@ void interfaceImpl_t::loadPlugins(const string &path)
 	list<string> plugins=listDir(path);
 	for(list<string>::iterator i=plugins.begin();i!=plugins.end();++i)
 	{
-		void *handle=dlopen(i->c_str(),RTLD_NOW);
-		if(handle==NULL)
-		{
-			cerr<<"dlerror: "<<dlerror()<<endl;
-			continue;
-		}
+		sharedlibrary_t plug(i->c_str());
+		if(!plug.isOpen()) continue;
 		reg_t *registerPlugin;
-#ifdef __APPLE__
-		registerPlugin=(reg_t *)dlsym(handle,"_registerPlugin");
-#else
-		registerPlugin=(reg_t *)dlsym(handle,"registerPlugin");
-#endif
-		if(registerPlugin==NULL)
-		{
-			cerr<<"dlerror: "<<dlerror()<<endl;
-			continue;
-		}
+		registerPlugin=(reg_t *)plug.getSymbol("registerPlugin");
+		if(registerPlugin==NULL) continue;
 		registerPlugin(*this);
-		pluginHandlers.push_back(handle);
+		pluginHandlers.push_back(plug);
 	}
 }
 
@@ -139,26 +117,27 @@ void  interfaceImpl_t::transformPop()
 void  interfaceImpl_t::addTexture(paramMap_t &params)
 {
 	texture_t *ntex=NULL;;
-	string name,type;
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 
 	params.getParam("name",name);
 	params.getParam("type",type);
-	if(name=="") return;
+	if(*name=="") return;
 
-	map<string,texture_factory_t *>::iterator i=texture_factory.find(type);
+	map<string,texture_factory_t *>::iterator i=texture_factory.find(*type);
 	if(i!=texture_factory.end()) 
 		ntex=i->second(params,*this);
-	else WARNING<<"Texture "<<type<<" not found"<<endl;
+	else WARNING<<"Texture "<<*type<<" not found"<<endl;
 
 	params.checkUnused("texture");
 
 	if (ntex==NULL) return;
-	if(texture_table.find(name)!=texture_table.end())
+	if(texture_table.find(*name)!=texture_table.end())
 	{
 		WARNING<<"Texture "<<name<<" redefined\n";
-		delete texture_table[name];
+		delete texture_table[*name];
 	}
-	texture_table[name]=ntex;
+	texture_table[*name]=ntex;
 
 }
 
@@ -166,39 +145,40 @@ void  interfaceImpl_t::addTexture(paramMap_t &params)
 
 void interfaceImpl_t::addShader(paramMap_t &params,list<paramMap_t> &lparams)
 {
-	string name,type;
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 	shader_t *ns=NULL;
 
 	params.getParam("name",name);
 	params.getParam("type",type);
-	if(name=="") return;
+	if(*name=="") return;
 
-	map<string,shader_factory_t *>::iterator i=shader_factory.find(type);
+	map<string,shader_factory_t *>::iterator i=shader_factory.find(*type);
 	if(i!=shader_factory.end()) 
 		ns=i->second(params,lparams,*this);
 	else
 	{
-		WARNING<<"Unknown shader type "<<type<<endl;
+		WARNING<<"Unknown shader type "<<*type<<endl;
 		return;
 	}
 	
-	if(ns==NULL) WARNING<<"Wrong shader definition for "<<name<<endl;
+	if(ns==NULL) WARNING<<"Wrong shader definition for "<<*name<<endl;
 	if(ns==NULL) return;
 	params.checkUnused("shader");
 
-	if(shader_table.find(name)!=shader_table.end())
+	if(shader_table.find(*name)!=shader_table.end())
 	{
-		WARNING<<"Shader "<<name<<" redefined\n";
-		delete shader_table[name];
+		WARNING<<"Shader "<<*name<<" redefined\n";
+		delete shader_table[*name];
 	}
-	shader_table[name]=ns;
-	INFO<<"Added shader "<<name<<endl;
+	shader_table[*name]=ns;
+	INFO<<"Added shader "<<*name<<endl;
 }
 
 
 void interfaceImpl_t::addObject_trimesh(const std::string &name,
-				std::vector<point3d_t> *verts, const std::vector<int> &faces,
-				std::vector<GFLOAT> *facesuv,std::vector<CFLOAT> *vcol,
+				std::vector<point3d_t> &verts, const std::vector<int> &faces,
+				std::vector<GFLOAT> &facesuv,std::vector<CFLOAT> &vcol,
 				const std::vector<std::string> &shaders,const std::vector<int> &faceshader,
 				float sm_angle,bool castShadows,bool useR,bool receiveR,bool caus,bool has_orco,
 				const color_t &caus_rcolor,const color_t &caus_tcolor,float caus_IOR)
@@ -222,37 +202,38 @@ void interfaceImpl_t::addObject_trimesh(const std::string &name,
 	}
 
 	meshObject_t *obj;
-	vector<triangle_t> *rfaces=new vector<triangle_t>;
+	vector<triangle_t> rfaces;
 
 	for(vector<int>::const_iterator i=faces.begin();i<faces.end();i+=3)
 	{
-		if((i[0]>(int)verts->size()) || (i[1]>(int)verts->size()) || 
-				(i[2]>(int)verts->size()))
+		if((i[0]>(int)verts.size()) || (i[1]>(int)verts.size()) || 
+				(i[2]>(int)verts.size()))
 			WARNING<<"Skiping face with verts out of bounds\n";
-		rfaces->push_back(triangle_t( &( (*verts)[i[0]] ),&( (*verts)[i[1]] ),
-					&( (*verts)[i[2]] ) ) );
+		rfaces.push_back(triangle_t( &( verts[i[0]] ),&( verts[i[1]] ),
+					&( verts[i[2]] ) ) );
 	}
-	if( facesuv!=NULL )
+	if(facesuv.size() >= (rfaces.size()*6) )
 	{
-		for(unsigned int i=0;i<rfaces->size();++i)
-			(*rfaces)[i].setUV(facesuv->begin()+6*i);
+		for(unsigned int i=0;i<rfaces.size();++i)
+			rfaces[i].setUV(facesuv.begin()+6*i);
 	}
-	if( vcol!=NULL )
+	if(vcol.size() >= (rfaces.size()*9) )
 	{
-		for(unsigned int i=0;i<rfaces->size();++i)
-			(*rfaces)[i].setVCOL(vcol->begin()+9*i);
+		for(unsigned int i=0;i<rfaces.size();++i)
+			rfaces[i].setVCOL(vcol.begin()+9*i);
 	}
 
-	if(faceshader.size()==rfaces->size())
+	if(faceshader.size()==rfaces.size())
 		for(unsigned int i=0;i<faceshader.size();++i)
-			(*rfaces)[i].setShader(shader_pointer[faceshader[i]]);
+			rfaces[i].setShader(shader_pointer[faceshader[i]]);
 
-	obj=new meshObject_t(M,verts,NULL,rfaces,facesuv,vcol);
+	vector<vector3d_t> normals;
+	obj=meshObject_t::factory(verts,normals,rfaces,facesuv,vcol);
 	obj->hasOrco(has_orco);
+	obj->transform(M);
 	if(sm_angle>0.0)
 		obj->autoSmooth(sm_angle);
 
-
 	if(object_table.find(name)!=object_table.end())
 	{
 		WARNING<<"Object "<<name<<" redefined\n";
@@ -276,7 +257,7 @@ void interfaceImpl_t::addObject_reference(const string &name,const string &origi
 			|| (original==name))
 		WARNING<<"Object "<<name<<" undefined\n";
 	else
-		obj=new referenceObject_t(M,object_table[original]);
+		obj=referenceObject_t::factory(M,object_table[original]);
 	
 	if(obj==NULL) return;
 	if(object_table.find(name)!=object_table.end())
@@ -289,7 +270,8 @@ void interfaceImpl_t::addObject_reference(const string &name,const string &origi
 
 void interfaceImpl_t::addLight(paramMap_t &params)
 {
-	string name,type;
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 	bool render=true,indirect=true;
 	light_t *l=NULL;
 
@@ -298,9 +280,9 @@ void interfaceImpl_t::addLight(paramMap_t &params)
 	params.getParam("use_in_render",render);
 	params.getParam("use_in_indirect",indirect);
 
-	if(name=="") return;
+	if(*name=="") return;
 
-	map<string,light_factory_t *>::iterator i=light_factory.find(type);
+	map<string,light_factory_t *>::iterator i=light_factory.find(*type);
 	if(i!=light_factory.end()) 
 		l=i->second(params,*this);
 
@@ -311,71 +293,81 @@ void interfaceImpl_t::addLight(paramMap_t &params)
 	l->useInRender(render);
 	l->useInIndirect(indirect);
 
-	if(light_table.find(name)!=light_table.end())
+	if(light_table.find(*name)!=light_table.end())
 	{
-		WARNING<<"Light "<<name<<" redefined\n";
-		delete light_table[name];
+		WARNING<<"Light "<<*name<<" redefined\n";
+		delete light_table[*name];
 	}
-	light_table[name]=l;
-	INFO<<"Added "<<type<<" light "<<name<<endl;
+	light_table[*name]=l;
+	INFO<<"Added "<<*type<<" light "<<*name<<endl;
 
 }
 
 void interfaceImpl_t::addCamera(paramMap_t &params)
 {
-	string name;
-	point3d_t from(0,1,0),to(0,0,0),up(0,1,1);
-	int resx=320,resy=200;
-	float dfocal=1.0;
-	params.getParam("name",name);
-	params.getParam("from",from);
-	params.getParam("to",to);
-	params.getParam("up",up);
-	params.getParam("resx",resx);
-	params.getParam("resy",resy);
-	params.getParam("focal",dfocal);
-	if(name=="") return;
-	camera_t *cam=new camera_t(from,to,up,resx,resy,dfocal);
-	if(camera_table.find(name)!=camera_table.end())
+	string _name, _type="perspective";
+	const string *name=&_name, *type=&_type;
+	point3d_t from(0,1,0), to(0,0,0), up(0,1,1);
+	int resx=320, resy=200;
+	float dfocal=1, apt=0, dofd=0;
+	bool useq = false;
+	params.getParam("name", name);
+	params.getParam("from", from);
+	params.getParam("to", to);
+	params.getParam("up", up);
+	params.getParam("resx", resx);
+	params.getParam("resy", resy);
+	params.getParam("focal", dfocal);
+	params.getParam("aperture", apt);
+	params.getParam("dof_distance", dofd);
+	params.getParam("use_qmc", useq);
+	params.getParam("type", type);
+	if (*name=="") return;
+	cameraType ct = CAM_PERSPECTIVE;
+	if (*type=="ortho") ct = CAM_ORTHO;
+	camera_t *cam=new camera_t(from, to, up, resx, resy, dfocal, apt, dofd, useq, ct);
+	if (camera_table.find(*name)!=camera_table.end())
 	{
-		WARNING<<"Camera "<<name<<" redefined\n";
-		delete camera_table[name];
+		WARNING << "Camera " << name << " redefined\n";
+		delete camera_table[*name];
 	}
-	camera_table[name]=cam;
-	INFO<<"Added camera "<<name<<endl;
+	camera_table[*name] = cam;
+	INFO << "Added camera " << *name << endl;
 
 }
 
 
 void interfaceImpl_t::addFilter(paramMap_t &params)
 {
-	string name,type;
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 	filter_t *f=NULL;
 
 	params.getParam("name",name);
 	params.getParam("type",type);
 
-	if(name=="") return;
+	if(*name=="") return;
 
-	if(type=="dof") 	f=filter_dof(params);
-	if(type=="antinoise") f=filter_antinoise(params);
+	if(*type=="dof") 	f=filter_dof(params);
+	if(*type=="antinoise") f=filter_antinoise(params);
 
 	params.checkUnused("filter");
 
 	if(f==NULL) return;
 
-	if(filter_table.find(name)!=filter_table.end()) 
+	if(filter_table.find(*name)!=filter_table.end()) 
 	{
-		WARNING<<"Filter "<<name<<" redefined\n";
-		delete filter_table[name];
+		WARNING<<"Filter "<<*name<<" redefined\n";
+		delete filter_table[*name];
 	}
-	filter_table[name]=f;
+	filter_table[*name]=f;
 
-	INFO<<"Added "<<type<<" filter "<<name<<endl;
+	INFO<<"Added "<<*type<<" filter "<<*name<<endl;
 }
 
 filter_t * interfaceImpl_t::filter_dof(paramMap_t &params)
 {
+	/*
 	GFLOAT focus=1.0;
 	GFLOAT near_radius=1.0;
 	GFLOAT far_radius=1.0,scale=1.0;
@@ -384,34 +376,40 @@ filter_t * interfaceImpl_t::filter_dof(paramMap_t &params)
 	params.getParam("far_blur",far_radius);
 	params.getParam("scale",scale);
 	return new filterDOF_t(focus,near_radius,far_radius,scale);
+	*/
+	return NULL;
 }
 
 filter_t *interfaceImpl_t::filter_antinoise(paramMap_t &params)
 {
+	/*
 	GFLOAT radius=1.0;
 	GFLOAT delta=1.0;
 	params.getParam("radius",radius);
 	params.getParam("max_delta",delta);
 	return new filterAntiNoise_t(radius,delta);
+	*/
+	return NULL;
 }
 
 
 void interfaceImpl_t::addBackground(paramMap_t &params)
 {
-	string name, type="";
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 	background_t *b=NULL;
 
 	params.getParam("name",name);
 	params.getParam("type",type);
 
-	if (name=="") return;
+	if (*name=="") return;
 
-	map<string,background_factory_t *>::iterator i=background_factory.find(type);
+	map<string,background_factory_t *>::iterator i=background_factory.find(*type);
 	if(i!=background_factory.end()) 
 		b=i->second(params,*this);
 	else
 	{
-		WARNING << "Wrong type for background " << type << endl;
+		WARNING << "Wrong type for background " << *type << endl;
 		return;
 	}
 
@@ -419,23 +417,23 @@ void interfaceImpl_t::addBackground(paramMap_t &params)
 
 	if(b==NULL) return;
 
-	if(background_table.find(name)!=background_table.end())
+	if(background_table.find(*name)!=background_table.end())
 	{
-		WARNING << "background " << name << " redefined\n";
-		delete background_table[name];
+		WARNING << "background " << *name << " redefined\n";
+		delete background_table[*name];
 	}
-	background_table[name]=b;
+	background_table[*name]=b;
 
-	INFO << "Added " << type << " background " << name << endl;
+	INFO << "Added " << *type << " background " << *name << endl;
 }
 
 
 void interfaceImpl_t::render(paramMap_t &params)
 {
-	string camera, outfile="salida.tga";
+	string _camera, _outfile="salida.tga",_background;
+	const string *camera=&_camera, *outfile=&_outfile,*background=&_background;
 	int raydepth = 3;
 	PFLOAT bias = 0.1;
-	string background = "";
 	bool save_alpha = false;
 
 	params.getParam("camera_name", camera);
@@ -480,14 +478,14 @@ void interfaceImpl_t::render(paramMap_t &params)
 	// single pass maximum samples value
 	params.getParam("AA_onepass_max", AA_onepass_max);
 
-	if(camera=="")
+	if(*camera=="")
 	{
 		ERRORMSG<<"No camera to render\n";
 		return;
 	}
-	if(camera_table.find(camera)==camera_table.end())
+	if(camera_table.find(*camera)==camera_table.end())
 	{
-		ERRORMSG<<"Camera "<<camera<<" does not exist\n";
+		ERRORMSG<<"Camera "<<*camera<<" does not exist\n";
 		return;
 	}
 
@@ -517,8 +515,10 @@ void interfaceImpl_t::render(paramMap_t &params)
 	color_t fgcol(1.0);
 	params.getParam("fog_color", fgcol);
 
-	scene_t scene;
-	camera_t *cam=camera_table[camera];
+	scene_t *pscene=scene_t::factory();
+	scene_t &scene=*pscene;
+
+	camera_t *cam=camera_table[*camera];
 
 	scene.setCamera(cam);
 	for(map<string,object3d_t *>::iterator i=object_table.begin();
@@ -531,7 +531,7 @@ void interfaceImpl_t::render(paramMap_t &params)
 			i!=filter_table.end();++i)
 		scene.addFilter((*i).second);
 
-	outTga_t out(cam->resX(), cam->resY(), outfile.c_str(), save_alpha);
+	outTga_t out(cam->resX(), cam->resY(), outfile->c_str(), save_alpha);
 
 	if (indir_sam)
 		cout << "Rendering with " << indir_sam << " indirect samples and "
@@ -548,12 +548,12 @@ void interfaceImpl_t::render(paramMap_t &params)
 
 	scene.setMaxRayDepth(raydepth);
 
-	if(background!="")
+	if(*background!="")
 	{
-		if(background_table.find(background)==background_table.end())
-			WARNING<<"Background "<<background<<" does not exist\n";
+		if(background_table.find(*background)==background_table.end())
+			WARNING<<"Background "<<*background<<" does not exist\n";
 		else
-			scene.setBackground(background_table[background]);
+			scene.setBackground(background_table[*background]);
 	}
 
 	// indirect values
@@ -570,17 +570,20 @@ void interfaceImpl_t::render(paramMap_t &params)
 	scene.setBias(bias);
 	if(cachedPathLight) scene.setRepeatFirst();
 
-	scene.render(out, cpus);
+        scene.setCPUs(cpus);
+        scene.render(out);
 
 	out.flush();
+
+	delete pscene;
 }
 
 void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 {
-	string camera;
+	string _camera,_background;
+	const string *camera=&_camera,*background=&_background;
 	int raydepth = 3;
 	PFLOAT bias = 0.1;
-	string background = "";
 
 	params.getParam("camera_name", camera);
 	params.getParam("raydepth", raydepth);
@@ -622,14 +625,14 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 	// single pass maximum samples value
 	params.getParam("AA_onepass_max", AA_onepass_max);
 
-	if(camera=="")
+	if(*camera=="")
 	{
 		ERRORMSG<<"No camera to render\n";
 		return;
 	}
-	if(camera_table.find(camera)==camera_table.end())
+	if(camera_table.find(*camera)==camera_table.end())
 	{
-		ERRORMSG<<"Camera "<<camera<<" does not exist\n";
+		ERRORMSG<<"Camera "<<*camera<<" does not exist\n";
 		return;
 	}
 
@@ -659,8 +662,9 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 	color_t fgcol(1.0);
 	params.getParam("fog_color", fgcol);
 
-	scene_t scene;
-	camera_t *cam=camera_table[camera];
+	scene_t *pscene=scene_t::factory();
+	scene_t &scene=*pscene;
+	camera_t *cam=camera_table[*camera];
 
 	scene.setCamera(cam);
 	for(map<string,object3d_t *>::iterator i=object_table.begin();
@@ -689,12 +693,12 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 
 	scene.setMaxRayDepth(raydepth);
 
-	if(background!="")
+	if(*background!="")
 	{
-		if(background_table.find(background)==background_table.end())
-			WARNING<<"Background "<<background<<" does not exist\n";
+		if(background_table.find(*background)==background_table.end())
+			WARNING<<"Background "<<*background<<" does not exist\n";
 		else
-			scene.setBackground(background_table[background]);
+			scene.setBackground(background_table[*background]);
 	}
 
 	// indirect values
@@ -711,7 +715,8 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 	scene.setBias(bias);
 	if(cachedPathLight) scene.setRepeatFirst();
 
-	scene.render(output, cpus);
+        scene.setCPUs(cpus);
+        scene.render(output);
 
 	output.flush();
 }
@@ -763,7 +768,7 @@ void interfaceImpl_t::registerFactory(const std::string &name,background_factory
 extern "C"
 {
 
-yafrayInterface_t *getYafray(int cpus,const string &ppath)
+YAFRAYPLUGIN_EXPORT yafrayInterface_t *getYafray(int cpus,const string &ppath)
 {
 	return new interfaceImpl_t(cpus,ppath);
 }
