@@ -19,6 +19,7 @@
  *      Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *      
  */
+
 #include "photonlight.h"
 using namespace std;
 #include <algorithm>
@@ -44,15 +45,6 @@ void photon_t::position(const point3d_t &_pos,PFLOAT bias)
 }
 
 
-//------------------------------------------------------------------------------------------
-// CONE ANGLE RESTRICTED VECTOR (replacement for random/discreteVectorCone)
-inline vector3d_t HemiVec_CONE(const vector3d_t &nrm,
-				const vector3d_t &Ru, const vector3d_t &Rv,
-				PFLOAT cosang, PFLOAT z1, PFLOAT z2)
-{
-  PFLOAT t1=2.0*M_PI*z1, t2=1.0-(1.0-cosang)*z2;
-  return (Ru*cos(t1) + Rv*sin(t1))*sqrt(1.0-t2*t2) + nrm*t2;
-}
 //------------------------------------------------------------------------------------------
 
 void insert(hash3d_t<photoAccum_t> &h,const photonMark_t &e)
@@ -104,6 +96,7 @@ photonLight_t::photonLight_t(const point3d_t &f,const point3d_t _to,PFLOAT angle
 	    base = nextPrime(base);
 	  }
 	}
+	use_in_indirect=false;
 }
 
 void photonLight_t::shoot_photon_caustic(scene_t &scene,photon_t &photon,
@@ -114,6 +107,8 @@ void photonLight_t::shoot_photon_caustic(scene_t &scene,photon_t &photon,
 	surfacePoint_t sp;
 	if(!scene.firstHit(nullstate,sp,photon.position(),dir)) {depth--;return;}
 	dis+=sp.Z();
+	const void *oldorigin=nullstate.skipelement;
+	nullstate.skipelement=sp.getOrigin();
 	
 	if(! (sp.getObject())->caustics() )
 	{
@@ -151,7 +146,7 @@ void photonLight_t::shoot_photon_caustic(scene_t &scene,photon_t &photon,
 			}
 		}
 	}
-
+	nullstate.skipelement=oldorigin;
 	depth--;
 }
 
@@ -162,6 +157,9 @@ void photonLight_t::shoot_photon_diffuse(scene_t &scene,photon_t &photon,
 	surfacePoint_t sp;
 	if(!scene.firstHit(nullstate,sp,photon.position(),dir)) {depth--;return;}
 	dis+=sp.Z();
+	const void *oldorigin=nullstate.skipelement;
+	nullstate.skipelement=sp.getOrigin();
+
 	photon.position(sp.P(),bias);
 	const shader_t *sha= sp.getShader();
 	vector3d_t edir=photon.lastPosition()-photon.position();
@@ -187,12 +185,13 @@ void photonLight_t::shoot_photon_diffuse(scene_t &scene,photon_t &photon,
 		else { r1=ourRandom();  r2=ourRandom(); }
 		//vector3d_t refDir = randomVectorCone(Ng,0.05,r1,r2);
 		//vector3d_t refDir = HemiVec_COSW(Ng, sp.NU(), sp.NV(), r1, r2);
- 		vector3d_t refDir = HemiVec_CONE(Ng, sp.NU(), sp.NV(), 0.05, r1, r2);
+ 		vector3d_t refDir = randomVectorCone(Ng, sp.NU(), sp.NV(), 0.05, r1, r2);
 		color_t newcolor=sha->fromRadiosity(nullstate,sp,ene,refDir);
 		photon.color(newcolor);
 		shoot_photon_diffuse(scene,photon,refDir,dis);
 	}
 
+	nullstate.skipelement=oldorigin;
 	depth--;
 }
 
@@ -409,7 +408,7 @@ void photonLight_t::init(scene_t &scene)
 		PFLOAT r1, r2;
 		if (use_QMC) { r1=HSEQ[0].getNext();  r2=((PFLOAT)i+0.5)/(PFLOAT)Np; }
 		else { r1=ourRandom();  r2=ourRandom(); }
-		dir = HemiVec_CONE(light_dir, LU, LV, angle_cos, r1, r2);
+		dir = randomVectorCone(light_dir, LU, LV, angle_cos, r1, r2);
 		if (dir.null()) continue;
 		if (mode==CAUSTIC) shoot_photon_caustic(scene, photon, dir);
 		if (mode==DIFFUSE) shoot_photon_diffuse(scene, photon, dir);
@@ -631,7 +630,8 @@ light_t *photonLight_t::factory(paramMap_t &params,renderEnvironment_t &render)
 	PFLOAT bias=0.001,disp=50.0,fr=1,cluster=1;
 	int nphotons=5000,search=50,depth=3,mindepth=1;
 	int mode=CAUSTIC;
-	string smode;
+	string _smode;
+	const string *smode=&_smode;
 	bool useqmc=false;
 
 	params.getParam("from", from);
@@ -651,7 +651,7 @@ light_t *photonLight_t::factory(paramMap_t &params,renderEnvironment_t &render)
 	if(!params.getParam("fixedradius",fr))
 		WARNING<<"Missing fixedradius, using default won't work.\n";
 
-	if(smode=="diffuse") mode=DIFFUSE;
+	if(*smode=="diffuse") mode=DIFFUSE;
 
 	if(!params.getParam("cluster",cluster))
 	{
@@ -691,7 +691,7 @@ pluginInfo_t photonLight_t::info()
 extern "C"
 {
 	
-void registerPlugin(renderEnvironment_t &render)
+YAFRAYPLUGIN_EXPORT void registerPlugin(renderEnvironment_t &render)
 {
 	render.registerFactory("photonlight",photonLight_t::factory);
 	std::cout<<"Registered photonlight\n";
