@@ -19,51 +19,29 @@
  *      Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  */
+
 #include "render.h"
 
 #include "sphere.h"
 #include "reference.h"
+#include "threadedscene.h"
+#include "forkedscene.h"
 
 
 #define WARNING cerr<<"[Warning]: "
 #define ERRORMSG cerr<<"[Error]: "
 #define INFO cerr<<"[Loader]: "
 
-#include<sys/types.h>
-#include<sys/stat.h>
-#include<dirent.h>
-#include <dlfcn.h>
+#include "yafsystem.h"
 
 __BEGIN_YAFRAY
 
-list<string> listDir(const string &dir)
-{
-	DIR *directorio;
-	struct dirent *entrada;
-	struct stat estado;
-	list<string> lista;
-
-	directorio=opendir(dir.c_str());
-	if(directorio==NULL) return lista;
-
-	entrada=readdir(directorio);
-	while(entrada!=NULL) 
-	{
-		string full=dir+"/"+entrada->d_name;
-		stat(full.c_str(),&estado);
-		if(S_ISREG(estado.st_mode))
-			lista.push_back(full);
-    entrada=readdir(directorio);
-  }
-  closedir(directorio);
-	return lista;
-}
-
-render_t::render_t(int ncpus,const string &pluginpath):M(1.0)
+render_t::render_t(int ncpus, strategy_t strat, const string &pluginpath):M(1.0)
 {
 	for(int i=0;i<MAX_AST;++i)
 		handler[i]=NULL;
 	cpus=ncpus;
+        strategy = strat;
 	scymin=scxmin=-2;
 	scymax=scxmax=2;
 	handler[AST_LITEM]=&render_t::itemList;
@@ -99,9 +77,6 @@ render_t::~render_t()
 #undef FREEMAP
 #undef MAPOF
 
-	for(list<void *>::iterator i=pluginHandlers.begin();i!=pluginHandlers.end();
-			++i)
-		dlclose(*i);
 }
 
 void render_t::loadPlugins(const string &path)
@@ -109,27 +84,16 @@ void render_t::loadPlugins(const string &path)
 	typedef void (reg_t)(renderEnvironment_t &);
 	cout<<"Loading plugins ..."<<endl;
 	list<string> plugins=listDir(path);
+	
 	for(list<string>::iterator i=plugins.begin();i!=plugins.end();++i)
 	{
-		void *handle=dlopen(i->c_str(),RTLD_NOW);
-		if(handle==NULL)
-		{
-			cerr<<"dlerror: "<<dlerror()<<endl;
-			continue;
-		}
+		sharedlibrary_t plug(i->c_str());
+		if(!plug.isOpen()) continue;
 		reg_t *registerPlugin;
-#ifdef __APPLE__
-		registerPlugin=(reg_t *)dlsym(handle,"_registerPlugin");
-#else
-		registerPlugin=(reg_t *)dlsym(handle,"registerPlugin");
-#endif
-		if(registerPlugin==NULL)
-		{
-			cerr<<"dlerror: "<<dlerror()<<endl;
-			continue;
-		}
+		registerPlugin=(reg_t *)plug.getSymbol("registerPlugin");
+		if(registerPlugin==NULL) continue;
 		registerPlugin(*this);
-		pluginHandlers.push_back(handle);
+		pluginHandlers.push_back(plug);
 	}
 }
 
@@ -174,29 +138,30 @@ void * render_t::texture(ast_t *ast)
 {
 	generic_data_t *tex=(generic_data_t *)ast;
 	texture_t *ntex=NULL;;
-	string name,type;
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 	paramMap_t &params=tex->params;
 
 	params.getParam("name",name);
 	params.getParam("type",type);
-	if(name=="") return NULL;
+	if(*name=="") return NULL;
 
-	map<string,texture_factory_t *>::iterator i=texture_factory.find(type);
+	map<string,texture_factory_t *>::iterator i=texture_factory.find(*type);
 	if(i!=texture_factory.end()) 
 		ntex=i->second(params,*this);
-	else WARNING<<"Texture "<<type<<" not found"<<endl;
+	else WARNING<<"Texture "<<*type<<" not found"<<endl;
 
 	params.checkUnused("texture");
 
 	if (ntex==NULL) return NULL;
-	if(texture_table.find(name)!=texture_table.end())
+	if(texture_table.find(*name)!=texture_table.end())
 	{
-		WARNING<<"Texture "<<name<<" redefined\n";
-		delete texture_table[name];
+		WARNING<<"Texture "<<*name<<" redefined\n";
+		delete texture_table[*name];
 	}
-	texture_table[name]=ntex;
+	texture_table[*name]=ntex;
 
-	INFO<<"Added texture "<<name<<endl;
+	INFO<<"Added texture "<<*name<<endl;
 	return ntex;
 }
 
@@ -264,37 +229,38 @@ void * render_t::shader(ast_t *ast)
 	shader_data_t *shader=(shader_data_t *)ast;
 	paramMap_t &params=shader->params;
 
-	string name,type;
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 	shader_t *ns=NULL;
 
 	params.getParam("name",name);
 	params.getParam("type",type);
-	if(name=="") return NULL;
+	if(*name=="") return NULL;
 
 	list<paramMap_t> lparams;
 	for(list<generic_data_t *>::iterator i=shader->lmod->l.begin();
 			i!=shader->lmod->l.end();++i)
 		lparams.push_back((*i)->params);
 
-	map<string,shader_factory_t *>::iterator i=shader_factory.find(type);
-	if(i!=shader_factory.end()) 
-		ns=i->second(params,lparams,*this);
+	map<string,shader_factory_t *>::iterator oi=shader_factory.find(*type);
+	if(oi!=shader_factory.end()) 
+		ns=oi->second(params,lparams,*this);
 	else
 	{
-		WARNING<<"Unknown shader type "<<type<<endl;
+		WARNING<<"Unknown shader type "<<*type<<endl;
 		return NULL;
 	}
 	
-	if(ns==NULL) WARNING<<"Wrong shader definition for "<<name<<endl;
+	if(ns==NULL) WARNING<<"Wrong shader definition for "<<*name<<endl;
 	if(ns==NULL) return NULL;
 	params.checkUnused("shader");
-	if(shader_table.find(name)!=shader_table.end())
+	if(shader_table.find(*name)!=shader_table.end())
 	{
-		WARNING<<"Shader "<<name<<" redefined\n";
-		delete shader_table[name];
+		WARNING<<"Shader "<<*name<<" redefined\n";
+		delete shader_table[*name];
 	}
-	shader_table[name]=ns;
-	INFO<<"Added shader "<<name<<endl;
+	shader_table[*name]=ns;
+	INFO<<"Added shader "<<*name<<endl;
 	return ns;
 
 }
@@ -705,7 +671,7 @@ void * render_t::object(ast_t *ast)
 				|| (object->original==object->name))
 			WARNING<<"Object "<<object->name<<" undefined\n";
 		else
-			obj=new referenceObject_t(M,object_table[object->original]);
+			obj=referenceObject_t::factory(M,object_table[object->original]);
 	}
 	else
 		obj=(object3d_t *)call(object->geometry);
@@ -735,7 +701,7 @@ void * render_t::object(ast_t *ast)
 void * render_t::mesh(ast_t *ast)
 {
 	mesh_data_t *mesh=(mesh_data_t *)ast;
-	vector<triangle_t> &faces=*(mesh->faces->faces);
+	vector<triangle_t> &faces=mesh->faces->faces;
 	vector<shader_t *> shaders;
 	for(vector<string>::iterator i=mesh->faces->shaders.begin();
 			i!=mesh->faces->shaders.end();++i)
@@ -750,17 +716,17 @@ void * render_t::mesh(ast_t *ast)
 	for(vector<triangle_t>::iterator i=faces.begin();i!=faces.end();++i)
 	{
 		int n=(int)((*i).a);
-		if((n>=(int)mesh->points->points->size()) || (n<0))
+		if((n>=(int)mesh->points->points.size()) || (n<0))
 			{ WARNING<<"Point "<<n<<" out of bounds in object\n"; n=0; }
-		(*i).a=&(*(mesh->points->points))[n];
+		(*i).a=&(mesh->points->points)[n];
 		n=(int)((*i).b);
-		if((n>=(int)mesh->points->points->size()) || (n<0))
+		if((n>=(int)mesh->points->points.size()) || (n<0))
 			{ WARNING<<"Point "<<n<<" out of bounds in object\n"; n=0; }
-		(*i).b=&(*(mesh->points->points))[n];
+		(*i).b=&(mesh->points->points)[n];
 		n=(int)((*i).c);
-		if((n>=(int)mesh->points->points->size()) || (n<0))
+		if((n>=(int)mesh->points->points.size()) || (n<0))
 			{ WARNING<<"Point "<<n<<" out of bounds in object\n"; n=0; }
-		(*i).c=&(*(mesh->points->points))[n];
+		(*i).c=&(mesh->points->points)[n];
 		
 		(*i).recNormal();
 		int ishader=(int)(*i).getShader();
@@ -770,40 +736,46 @@ void * render_t::mesh(ast_t *ast)
 			(*i).setShader(shaders[ishader]);
 	}
 	int uv_offset=0, vcol_offset=0;
-	for(vector<triangle_t>::iterator i=mesh->faces->faces->begin();
-				i!=mesh->faces->faces->end();++i) {
+	for(vector<triangle_t>::iterator i=mesh->faces->faces.begin();
+				i!=mesh->faces->faces.end();++i) {
 		if(i->hasuv)
 		{
-			i->uv = mesh->faces->facesuv->begin() + uv_offset;
+			i->uv = mesh->faces->facesuv.begin() + uv_offset;
 			uv_offset += 6;
 		}
 		if(i->has_vcol)
 		{
-			i->vcol = mesh->faces->faces_vcol->begin() + vcol_offset;
+			i->vcol = mesh->faces->faces_vcol.begin() + vcol_offset;
 			vcol_offset += 9;
 		}
 	}
-	meshObject_t *obj = new meshObject_t(M, mesh->points->points, NULL,
+	vector<vector3d_t> normals;
+	meshObject_t *obj = meshObject_t::factory(mesh->points->points, normals,
 						mesh->faces->faces, mesh->faces->facesuv, mesh->faces->faces_vcol);
 
+	obj->hasOrco(mesh->orco);
+	obj->transform(M);
 	if(mesh->autosmooth)
 		obj->autoSmooth(mesh->angle);
-	obj->hasOrco(mesh->orco);
 	return obj;
 }
 
 void * render_t::sphere(ast_t *ast)
 {
+	/*
 	sphere_data_t *sphere=(sphere_data_t *)ast;
 	sphere_t *sp=new sphere_t(M*(sphere->center),sphere->radius);
 	return sp;
+	*/
+	return NULL;
 }
 
 
 void * render_t::light(ast_t *ast)
 {
 	generic_data_t *light=(generic_data_t *)ast;
-	string name,type;
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 	bool render=true,indirect=true;
 	light_t *l=NULL;
 
@@ -812,9 +784,9 @@ void * render_t::light(ast_t *ast)
 	light->params.getParam("use_in_render",render);
 	light->params.getParam("use_in_indirect",indirect);
 
-	if(name=="") return NULL;
+	if(*name=="") return NULL;
 
-	map<string,light_factory_t *>::iterator i=light_factory.find(type);
+	map<string,light_factory_t *>::iterator i=light_factory.find(*type);
 	if(i!=light_factory.end()) 
 		l=i->second(light->params,*this);
 	
@@ -825,13 +797,13 @@ void * render_t::light(ast_t *ast)
 	l->useInRender(render);
 	l->useInIndirect(indirect);
 
-	if(light_table.find(name)!=light_table.end())
+	if(light_table.find(*name)!=light_table.end())
 	{
-		WARNING<<"Light "<<name<<" redefined\n";
-		delete light_table[name];
+		WARNING<<"Light "<<*name<<" redefined\n";
+		delete light_table[*name];
 	}
-	light_table[name]=l;
-	INFO<<"Added "<<type<<" light "<<name<<endl;
+	light_table[*name]=l;
+	INFO<<"Added "<<*type<<" light "<<*name<<endl;
 
 	return NULL;
 }
@@ -839,17 +811,19 @@ void * render_t::light(ast_t *ast)
 void * render_t::camera(ast_t *ast)
 {
 	camera_data_t *camera=(camera_data_t *)ast;
-	if(camera->name=="") return NULL;
+	if (camera->name=="") return NULL;
+	cameraType ct = CAM_PERSPECTIVE;
+	if (camera->camtype=="ortho") ct = CAM_ORTHO;
 	camera_t *cam=new camera_t(camera->from, camera->to, camera->up,
 														camera->resx, camera->resy,
-														camera->dfocal, camera->laperture, camera->dof_use_qmc);
-	if(camera_table.find(camera->name)!=camera_table.end())
+														camera->dfocal, camera->laperture, camera->dofdist, camera->dof_use_qmc, ct);
+	if (camera_table.find(camera->name)!=camera_table.end())
 	{
-		WARNING<<"Camera "<<camera->name<<" redefined\n";
+		WARNING << "Camera " << camera->name << " redefined\n";
 		delete camera_table[camera->name];
 	}
 	camera_table[camera->name]=cam;
-	INFO<<"Added camera "<<camera->name<<endl;
+	INFO << "Added camera " << camera->name << endl;
 	return cam;
 
 }
@@ -858,34 +832,36 @@ void * render_t::camera(ast_t *ast)
 void * render_t::filter(ast_t *ast)
 {
 	generic_data_t *filter=(generic_data_t *)ast;
-	string name,type;
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 	filter_t *f=NULL;
 
 	filter->params.getParam("name",name);
 	filter->params.getParam("type",type);
 
-	if(name=="") return NULL;
+	if(*name=="") return NULL;
 
-	if(type=="dof") 	f=filter_dof(filter->params);
-	if(type=="antinoise") f=filter_antinoise(filter->params);
+	if(*type=="dof") 	f=filter_dof(filter->params);
+	if(*type=="antinoise") f=filter_antinoise(filter->params);
 
 	filter->params.checkUnused("filter");
 
 	if(f==NULL) return NULL;
 
-	if(filter_table.find(name)!=filter_table.end()) 
+	if(filter_table.find(*name)!=filter_table.end()) 
 	{
-		WARNING<<"Filter "<<name<<" redefined\n";
-		delete filter_table[name];
+		WARNING<<"Filter "<<*name<<" redefined\n";
+		delete filter_table[*name];
 	}
-	filter_table[name]=f;
+	filter_table[*name]=f;
 
-	INFO<<"Added "<<type<<" filter "<<name<<endl;
+	INFO<<"Added "<<*type<<" filter "<<*name<<endl;
 	return f;
 }
 
 filter_t * render_t::filter_dof(paramMap_t &params)
 {
+	/*
 	GFLOAT focus=1.0;
 	GFLOAT near_radius=1.0;
 	GFLOAT far_radius=1.0,scale=1.0;
@@ -894,30 +870,36 @@ filter_t * render_t::filter_dof(paramMap_t &params)
 	params.getParam("far_blur",far_radius);
 	params.getParam("scale",scale);
 	return new filterDOF_t(focus,near_radius,far_radius,scale);
+	*/
+	return NULL;
 }
 
 filter_t *render_t::filter_antinoise(paramMap_t &params)
 {
+	/*
 	GFLOAT radius=1.0;
 	GFLOAT delta=1.0;
 	params.getParam("radius",radius);
 	params.getParam("max_delta",delta);
 	return new filterAntiNoise_t(radius,delta);
+	*/
+	return NULL;
 }
 
 
 void * render_t::background(ast_t *ast)
 {
 	generic_data_t *background=(generic_data_t *)ast;
-	string name, type="";
+	string _name,_type;
+	const string *name=&_name,*type=&_type;
 	background_t *b=NULL;
 
 	background->params.getParam("name",name);
 	background->params.getParam("type",type);
 
-	if (name=="") return NULL;
+	if (*name=="") return NULL;
 
-	map<string,background_factory_t *>::iterator i=background_factory.find(type);
+	map<string,background_factory_t *>::iterator i=background_factory.find(*type);
 	if(i!=background_factory.end()) 
 		b=i->second(background->params,*this);
 	else
@@ -930,21 +912,21 @@ void * render_t::background(ast_t *ast)
 	//	b = background_HDRI(background->params);
 	//else if (type=="image")
 	//	b = background_image(background->params);
-		WARNING << "Wrong type for background " << type << endl;
+		WARNING << "Wrong type for background " << *type << endl;
 		return NULL;
 	}
 	background->params.checkUnused("background");
 
 	if(b==NULL) return NULL;
 
-	if(background_table.find(name)!=background_table.end())
+	if(background_table.find(*name)!=background_table.end())
 	{
-		WARNING << "background " << name << " redefined\n";
-		delete background_table[name];
+		WARNING << "background " << *name << " redefined\n";
+		delete background_table[*name];
 	}
-	background_table[name]=b;
+	background_table[*name]=b;
 
-	INFO << "Added " << type << " background " << name << endl;
+	INFO << "Added " << *type << " background " << *name << endl;
 	return b;
 }
 
@@ -1039,10 +1021,10 @@ void * render_t::render(ast_t *ast)
 	generic_data_t *rend=(generic_data_t *)ast;
 	paramMap_t &params=rend->params;
 
-	string camera, outfile="salida.tga";
+	string _camera, _outfile="salida.tga",_background;
+	const string *camera=&_camera, *outfile=&_outfile, *background=&_background;
 	int raydepth = 3;
 	PFLOAT bias = 0.1;
-	string background = "";
 	bool save_alpha = false;
 
 	params.getParam("camera_name", camera);
@@ -1087,14 +1069,14 @@ void * render_t::render(ast_t *ast)
 	// single pass maximum samples value
 	params.getParam("AA_onepass_max", AA_onepass_max);
 
-	if(camera=="")
+	if(*camera=="")
 	{
 		ERRORMSG<<"No camera to render\n";
 		return NULL;
 	}
-	if(camera_table.find(camera)==camera_table.end())
+	if(camera_table.find(*camera)==camera_table.end())
 	{
-		ERRORMSG<<"Camera "<<camera<<" does not exist\n";
+		ERRORMSG<<"Camera "<<*camera<<" does not exist\n";
 		return NULL;
 	}
 
@@ -1124,21 +1106,38 @@ void * render_t::render(ast_t *ast)
 	color_t fgcol(1.0);
 	params.getParam("fog_color", fgcol);
 
-	scene_t scene;
-	camera_t *cam=camera_table[camera];
+	scene_t *scene;
+	switch (strategy) {
+		case MONO:
+			scene = scene_t::factory();
+			break;
+		case THREAD:
+			scene = scene_t::factory();
+			//scene = new threadedscene_t;
+			break;
+		case FORK:
+			scene = scene_t::factory();
+			//scene = new forkedscene_t;
+			break;
+		default:
+			scene = scene_t::factory();
+			break;
+	}
 
-	scene.setCamera(cam);
+	camera_t *cam=camera_table[*camera];
+
+        scene->setCamera(cam);
 	for(map<string,object3d_t *>::iterator i=object_table.begin();
 			i!=object_table.end();++i)
-		scene.addObject((*i).second);
+                scene->addObject((*i).second);
 	for(map<string,light_t *>::iterator i=light_table.begin();
 			i!=light_table.end();++i)
-		scene.addLight((*i).second);
+                scene->addLight((*i).second);
 	for(map<string,filter_t *>::iterator i=filter_table.begin();
 			i!=filter_table.end();++i)
-		scene.addFilter((*i).second);
+                scene->addFilter((*i).second);
 
-	outTga_t out(cam->resX(), cam->resY(), outfile.c_str(), save_alpha);
+	outTga_t out(cam->resX(), cam->resY(), outfile->c_str(), save_alpha);
 
 	if (indir_sam)
 		cout << "Rendering with " << indir_sam << " indirect samples and "
@@ -1153,35 +1152,38 @@ void * render_t::render(ast_t *ast)
 			<< AA_passes*AA_minsamples << " samples total.\n";
 	else cout << "No anti-aliasing.\n";
 
-	scene.setMaxRayDepth(raydepth);
+        scene->setMaxRayDepth(raydepth);
 
-	if(background!="")
+	if(*background!="")
 	{
-		if(background_table.find(background)==background_table.end())
-			WARNING<<"Background "<<background<<" does not exist\n";
+		if(background_table.find(*background)==background_table.end())
+			WARNING<<"Background "<<*background<<" does not exist\n";
 		else
-			scene.setBackground(background_table[background]);
+                        scene->setBackground(background_table[*background]);
 	}
 
 	// indirect values
-	scene.setIndirect(indir_sam, indir_pow);
+        scene->setIndirect(indir_sam, indir_pow);
 	// gamma & exposure
-	scene.setExposure(exposure);
-	scene.setGamma(gamma);
+        scene->setExposure(exposure);
+        scene->setGamma(gamma);
 	// fog
-	scene.setFog(fgdens, fgcol);
+        scene->setFog(fgdens, fgcol);
 
 	// set the AA params
-	scene.setAASamples(AA_passes, AA_minsamples, AA_pixelwidth, AA_threshold, AA_onepass_max);
+        scene->setAASamples(AA_passes, AA_minsamples, AA_pixelwidth, AA_threshold, AA_onepass_max);
 
-	scene.setBias(bias);
-	if(cachedPathLight) scene.setRepeatFirst();
+        scene->setBias(bias);
+        if(cachedPathLight) scene->setRepeatFirst();
 
-	scene.setRegion(scxmin,scxmax,scymin,scymax);
-	scene.render(out, cpus);
+        scene->setRegion(scxmin,scxmax,scymin,scymax);
+        scene->setCPUs(cpus);
+        scene->render(out);
 
 	out.flush();
 
+        delete scene;
+
 	return NULL;
 }
 
