@@ -1,3 +1,4 @@
+
 #include "basicblocks.h"
 
 using namespace std;
@@ -14,10 +15,11 @@ colorA_t floatToColor_t::stdoutColor(renderState_t &state,const surfacePoint_t &
 shader_t * floatToColor_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string inputs;
+	string _inputs;
+	const string *inputs=&_inputs;
 	shader_t *input=NULL;
 	bparams.getParam("input",inputs);
-	input=render.getShader(inputs);
+	input=render.getShader(*inputs);
 	if(input!=NULL)
 		return new floatToColor_t(input);
 	else return NULL;
@@ -26,10 +28,11 @@ shader_t * floatToColor_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lp
 shader_t * colorToFloat_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string inputs;
+	string _inputs;
+	const string *inputs=&_inputs;
 	shader_t *input=NULL;
 	bparams.getParam("input",inputs);
-	input=render.getShader(inputs);
+	input=render.getShader(*inputs);
 	if(input!=NULL)
 		return new colorToFloat_t(input);
 	else return NULL;
@@ -38,10 +41,11 @@ shader_t * colorToFloat_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lp
 shader_t * sinNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string in;
+	string _inputs;
+	const string *inputs=&_inputs;
 	shader_t *input=NULL;
-	bparams.getParam("input",in);
-	input=render.getShader(in);
+	bparams.getParam("input",inputs);
+	input=render.getShader(*inputs);
 	if(input!=NULL)
 		return new sinNode_t(input);
 	else return NULL;
@@ -50,14 +54,15 @@ shader_t * sinNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams
 shader_t * mulNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string in1,in2;
+	string _in1,_in2;
+	const string *in1=&_in1,*in2=&_in2;
 	shader_t *input1=NULL,*input2=NULL;
 	CFLOAT val=1.0;
 	bparams.getParam("input1",in1);
 	bparams.getParam("input2",in2);
 	bparams.getParam("value",val);
-	input1=render.getShader(in1);
-	input2=render.getShader(in2);
+	input1=render.getShader(*in1);
+	input2=render.getShader(*in2);
 	return new mulNode_t(input1,input2,val);
 }
 
@@ -65,11 +70,12 @@ shader_t * coordsNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lpar
 				        renderEnvironment_t &render)
 {
 	int w=0;
-	string var;
+	string _var;
+	const string *var=&_var;
 	bparams.getParam("coord",var);
-	if(var=="X") w=0;
-	if(var=="Y") w=1;
-	if(var=="Z") w=2;
+	if(*var=="X") w=0;
+	if(*var=="Y") w=1;
+	if(*var=="Z") w=2;
 	return new coordsNode_t(w);
 }
 // CLOUDS
@@ -105,7 +111,8 @@ colorA_t cloudsNode_t::stdoutColor(renderState_t &state,const surfacePoint_t &sp
 shader_t * cloudsNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string in1,in2;
+	string _in1,_in2;
+	const string *in1=&_in1,*in2=&_in2;
 	CFLOAT size=1.0;
 	int dep;
 	shader_t *input1=NULL,*input2=NULL;
@@ -113,8 +120,8 @@ shader_t * cloudsNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lpar
 	bparams.getParam("input2",in2);
 	bparams.getParam("size",size);
 	bparams.getParam("depth",dep);
-	input1=render.getShader(in1);
-	input2=render.getShader(in2);
+	input1=render.getShader(*in1);
+	input2=render.getShader(*in2);
 
 	return new cloudsNode_t(size,dep,input1,input2);
 }
@@ -150,7 +157,8 @@ colorA_t marbleNode_t::stdoutColor(renderState_t &state,const surfacePoint_t &sp
 shader_t * marbleNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string in1, in2;
+	string _in1,_in2;
+	const string *in1=&_in1,*in2=&_in2;
 	CFLOAT size = 1;
 	int dep = 2;
 	CFLOAT turb=1, shp=1;
@@ -165,8 +173,8 @@ shader_t * marbleNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lpar
 	bparams.getParam("sharpness", shp);
 	bparams.getParam("hard", hrd);
 
-	input1=render.getShader(in1);
-	input2=render.getShader(in2);
+	input1=render.getShader(*in1);
+	input2=render.getShader(*in2);
 
 	return new marbleNode_t(size, dep, turb, shp, hrd, input1, input2);
 }
@@ -202,7 +210,8 @@ colorA_t woodNode_t::stdoutColor(renderState_t &state,const surfacePoint_t &sp,
 shader_t * woodNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string in1, in2;
+	string _in1,_in2;
+	const string *in1=&_in1,*in2=&_in2;
 	CFLOAT size = 1;
 	int dep = 2;
 	CFLOAT turb = 1;
@@ -219,8 +228,8 @@ shader_t * woodNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparam
 	bparams.getParam("ringscale_y", ry);
 	bparams.getParam("hard", hrd);
 
-	input1=render.getShader(in1);
-	input2=render.getShader(in2);
+	input1=render.getShader(*in1);
+	input2=render.getShader(*in2);
 
 	return new woodNode_t(size, dep, turb, rx, ry, hrd, input1, input2);
 }
@@ -250,10 +259,11 @@ colorA_t colorBandNode_t::stdoutColor(renderState_t &state,const surfacePoint_t
 shader_t * colorBandNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string inputs;
+	string _inputs;
+	const string *inputs=&_inputs;
 	shader_t *input=NULL;
 	bparams.getParam("input",inputs);
-	input=render.getShader(inputs);
+	input=render.getShader(*inputs);
 	if(input==NULL)
 	{
 		cerr<<"Input shader for colorband not found\n";
@@ -326,7 +336,8 @@ void phongNode_t::getCaustics(renderState_t &state,const surfacePoint_t &sp,cons
 shader_t * phongNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string c,s,e,cr,ct;
+	string _c,_s,_e,_cr,_ct;
+	const string *c=&_c,*s=&_s,*e=&_e,*cr=&_cr,*ct=&_ct;
 	CFLOAT hard=1.0,ior=1.0;
 	shader_t *color=NULL,*specular=NULL,*env=NULL,*causr=NULL,*caust=NULL;
 	bparams.getParam("color",c);
@@ -336,17 +347,17 @@ shader_t * phongNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lpara
 	bparams.getParam("caus_tcolor",ct);
 	bparams.getParam("hard",hard);
 	bparams.getParam("IOR",ior);
-	color=render.getShader(c);
-	specular=render.getShader(s);
-	env=render.getShader(e);
-	causr=render.getShader(cr);
-	caust=render.getShader(ct);
+	color=render.getShader(*c);
+	specular=render.getShader(*s);
+	env=render.getShader(*e);
+	causr=render.getShader(*cr);
+	caust=render.getShader(*ct);
 
-	if((c!="") && (color==NULL)) cerr<<"Input shader "<<c<<" not found\n";
-	if((s!="") && (specular==NULL)) cerr<<"Input shader "<<s<<" not found\n";
-	if((e!="") && (env==NULL)) cerr<<"Input shader "<<e<<" not found\n";
-	if((cr!="") && (causr==NULL)) cerr<<"Input shader "<<cr<<" not found\n";
-	if((ct!="") && (caust==NULL)) cerr<<"Input shader "<<ct<<" not found\n";
+	if((*c!="") && (color==NULL)) cerr<<"Input shader "<<c<<" not found\n";
+	if((*s!="") && (specular==NULL)) cerr<<"Input shader "<<s<<" not found\n";
+	if((*e!="") && (env==NULL)) cerr<<"Input shader "<<e<<" not found\n";
+	if((*cr!="") && (causr==NULL)) cerr<<"Input shader "<<cr<<" not found\n";
+	if((*ct!="") && (caust==NULL)) cerr<<"Input shader "<<ct<<" not found\n";
 
 	return new phongNode_t(color,specular,env,causr,caust,hard,ior);
 }
@@ -371,15 +382,16 @@ colorA_t rgbNode_t::stdoutColor(renderState_t &state,const surfacePoint_t &sp,co
 shader_t * rgbNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string in1,in2,in3;
+	string _in1,_in2,_in3;
+	const string *in1=&_in1,*in2=&_in2,*in3=&_in3;
 	shader_t *inputred=NULL,*inputgreen=NULL,*inputblue=NULL;
 
 	bparams.getParam("inputred",in1);
 	bparams.getParam("inputgreen",in2);
 	bparams.getParam("inputblue",in3);
-	inputred=render.getShader(in1);
-	inputgreen=render.getShader(in2);
-	inputblue=render.getShader(in3);
+	inputred=render.getShader(*in1);
+	inputgreen=render.getShader(*in2);
+	inputblue=render.getShader(*in3);
 
 	color_t color(0.0);
 	bparams.getParam("color",color);
@@ -464,15 +476,16 @@ colorA_t hsvNode_t::stdoutColor(renderState_t &state,const surfacePoint_t &sp,
 shader_t * hsvNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string in1,in2,in3;
+	string _in1,_in2,_in3;
+	const string *in1=&_in1,*in2=&_in2,*in3=&_in3;
 	shader_t *inputhue=NULL,*inputsaturation=NULL,*inputvalue=NULL;
 
 	bparams.getParam("inputhue",in1);
 	bparams.getParam("inputsaturation",in2);
 	bparams.getParam("inputvalue",in3);
-	inputhue=render.getShader(in1);
-	inputsaturation=render.getShader(in2);
-	inputvalue=render.getShader(in3);
+	inputhue=render.getShader(*in1);
+	inputsaturation=render.getShader(*in2);
+	inputvalue=render.getShader(*in3);
 
 	CFLOAT hue=1.0,saturation=1.0,value=1.0;
 	bparams.getParam("hue",hue);
@@ -525,7 +538,14 @@ colorA_t coneTraceNode_t::stdoutColor(renderState_t &state,const surfacePoint_t
 	}
 	int oldlevel=state.rayDivision;
 
-	if((cosa==1.0) || (oldlevel>1)) return scene->raytrace(state,P, basedir)*color;
+	const void *oldorigin=state.skipelement;
+	state.skipelement=sp.getOrigin();
+	if((cosa==1.0) || (oldlevel>1)) 
+	{
+		color_t res=scene->raytrace(state,P, basedir)*color;
+		state.skipelement=oldorigin;
+		return res;
+	}
 
 	state.rayDivision=samples;
 	color_t res(0.0);
@@ -545,6 +565,7 @@ colorA_t coneTraceNode_t::stdoutColor(renderState_t &state,const surfacePoint_t
 		}
 	res*=div;
 	state.rayDivision=oldlevel;
+	state.skipelement=oldorigin;
 	return res*color;
 }
 
@@ -589,7 +610,8 @@ colorA_t fresnelNode_t::stdoutColor(renderState_t &state,const surfacePoint_t &s
 shader_t * fresnelNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string inR,inT;
+	string _inR,_inT;
+	const string *inR=&_inR,*inT=&_inT;
 	shader_t *inputR=NULL,*inputT=NULL;
 	PFLOAT ior=1.0;
 	CFLOAT minr=0.0;
@@ -597,20 +619,21 @@ shader_t * fresnelNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lpa
 	bparams.getParam("transmitted",inT);
 	bparams.getParam("IOR",ior);
 	bparams.getParam("min_refle",minr);
-	inputR=render.getShader(inR);
-	inputT=render.getShader(inT);
+	inputR=render.getShader(*inR);
+	inputT=render.getShader(*inT);
 	return new fresnelNode_t(inputR,inputT,ior,minr);
 }
 
 shader_t * imageNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string name;
+	string _name;
+	const string *name=&_name;
 	bparams.getParam("filename", name);
-	if (name=="")
+	if (*name=="")
 		cerr << "Required argument filename not found for image block\n";
 	else
-		return new imageNode_t(name.c_str());
+		return new imageNode_t(name->c_str());
 	return NULL;
 }
 
@@ -665,7 +688,8 @@ colorA_t goboNode_t::stdoutColor(renderState_t &state,const surfacePoint_t &sp,c
 shader_t * goboNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparams,
 				        renderEnvironment_t &render)
 {
-	string si1,si2,sgC,sgV;
+	string _si1,_si2,_sgC,_sgV;
+	const string *si1=&_si1,*si2=&_si2,*sgC=&_sgC,*sgV=&_sgV;
 	shader_t *i1=NULL,*i2=NULL,*gC=NULL,*gV=NULL;
 	bool hard = true;
 	CFLOAT ev = 0.5;
@@ -677,10 +701,10 @@ shader_t * goboNode_t::factory(paramMap_t &bparams,std::list<paramMap_t> &lparam
 	bparams.getParam("hardedge",hard);
 	bparams.getParam("edgeval",ev);
 
-	i1=render.getShader(si1);
-	i2=render.getShader(si2);
-	gV=render.getShader(sgC);
-	gC=render.getShader(sgV);
+	i1=render.getShader(*si1);
+	i2=render.getShader(*si2);
+	gV=render.getShader(*sgC);
+	gC=render.getShader(*sgV);
 
 	if(i1 == NULL) cerr << "Input Shader 1 -" << si1 << "- not found\n";
 	if(i2 == NULL) cerr << "Input Shader 2 -" << si2 << "- not found\n";
