@@ -1,3 +1,4 @@
+
 #include "basicblocks.h"
 #include "basictex.h"
 #include "imagebackg.h"
@@ -7,7 +8,7 @@ __BEGIN_YAFRAY
 extern "C"
 {
 	
-void registerPlugin(renderEnvironment_t &render)
+YAFRAYPLUGIN_EXPORT void registerPlugin(renderEnvironment_t &render)
 {
 	render.registerFactory("float2color",floatToColor_t::factory);
 	render.registerFactory("color2float",colorToFloat_t::factory);
