@@ -1,3 +1,4 @@
+
 #include "blendershader.h"
 
 __BEGIN_YAFRAY
@@ -67,7 +68,8 @@ bool blenderMapperNode_t::doMapping(const surfacePoint_t &sp, const vector3d_t &
 		case TXC_ORCO: {
 			// scaled/rotated/translated to fit object size/rotation/position
 			if (sp.hasOrco())
-				texpt = sp.getObject()->toObject(sp.orco());
+				texpt = sp.orco();
+				//texpt = sp.getObject()->toObject(sp.orco());
 			else
 				texpt = sp.getObject()->toObjectOrco(sp.P());
 			break;
@@ -305,9 +307,10 @@ using namespace std;
 shader_t * blenderMapperNode_t::factory(paramMap_t &bparams,
 		std::list<paramMap_t> &lparams,renderEnvironment_t &render)
 {
-	string childname;
+	string _childname;
+	const string *childname=&_childname;
 	bparams.getParam("input",childname);
-	shader_t *child=render.getShader(childname);
+	shader_t *child=render.getShader(*childname);
 	if(child==NULL)
 	{
 		cerr<<"Null input in blender mapper\n";
@@ -316,8 +319,12 @@ shader_t * blenderMapperNode_t::factory(paramMap_t &bparams,
 	blenderMapperNode_t *mapper=new blenderMapperNode_t(child);
 
 	GFLOAT size=1, sizex=1, sizey=1, sizez=1;
-	string mapping="flat", texco="orco", clipping="extend";	// defaults
-	string projx="x", projy="y", projz="z";
+	
+	string _mapping="flat", _texco="orco", _clipping="extend";	// defaults
+	string _projx="x", _projy="y", _projz="z";
+	const string *mapping=&_mapping, *texco=&_texco, *clipping=&_clipping;	// defaults
+	const string *projx=&_projx, *projy=&_projy, *projz=&_projz;
+
 	int xrep=1, yrep=1;
 	matrix4x4_t txm(1);
 	GFLOAT ofsx=0, ofsy=0, ofsz=0;
@@ -371,10 +378,10 @@ shader_t * blenderMapperNode_t::factory(paramMap_t &bparams,
 	mapper->ofsY(ofsy);
 	mapper->ofsZ(ofsz);
 	if (size!=1.0) mapper->size(size);
-	mapper->string2maptype(mapping);
-	mapper->string2texcotype(texco);
-	mapper->string2cliptype(clipping);
-	mapper->string2texprojection(projx, projy, projz);
+	mapper->string2maptype(*mapping);
+	mapper->string2texcotype(*texco);
+	mapper->string2cliptype(*clipping);
+	mapper->string2texprojection(*projx, *projy, *projz);
 	mapper->setRepeat(xrep, yrep);
 	mapper->setCrop(minx, miny, maxx, maxy);
 	if (hasmtx) mapper->setTransform(txm);
@@ -859,6 +866,9 @@ color_t blenderShader_t::fromLight(renderState_t &state, const surfacePoint_t &s
 color_t blenderShader_t::fromWorld(renderState_t &state, const surfacePoint_t &sp,
 																const scene_t &s, const vector3d_t &eye) const
 {
+	const void *oldorigin=state.skipelement;
+	state.skipelement=sp.getOrigin();
+
 	colorA_t Rresul(0.0), Tresul(0.0);
 	vector3d_t edir = eye;
 	edir.normalize();
@@ -961,6 +971,7 @@ color_t blenderShader_t::fromWorld(renderState_t &state, const surfacePoint_t &s
 		// vcol_paint has priority over vcol_light
 		if ((mat_mode & (MAT_VCOL_LIGHT | MAT_VCOL_PAINT))==MAT_VCOL_LIGHT) rc += col*(colorA_t)sp.vertex_col();
 	}
+	state.skipelement=oldorigin;
 	return rc;
 }
 
@@ -990,7 +1001,7 @@ void blenderShader_t::displace(renderState_t &state,surfacePoint_t &sp, const ve
 	{
 		for(vector<blenderModulator_t>::const_iterator ite=mods.begin();ite!=mods.end();++ite)
 		{
-			(*ite).blenderDisplace(state, sp, eye, res*state.traveled /*sp.Z()*/);
+			(*ite).blenderDisplace(state, sp, eye, res*state.traveled);
 		}
 	}
 }
@@ -1004,7 +1015,8 @@ shader_t * blenderShader_t::factory(paramMap_t &bparams,std::list<paramMap_t> &l
 	bool fast_fresnel=false,tir=false;
 	// new fresnel angle dependent colors
 	color_t reflected2(0.0), transmitted2(0.0);
-	string matmodes = "";
+	string _matmodes;
+	const string *matmodes = &_matmodes;
 
 	bparams.getParam("color", color);
 	bparams.getParam("specular_color", specular);
@@ -1030,29 +1042,34 @@ shader_t * blenderShader_t::factory(paramMap_t &bparams,std::list<paramMap_t> &l
 	blenderShader_t *ns = new blenderShader_t(color, specular, mirror, difref, specam, hard, alpha, emit,
 																						reflected, transmitted, reflected2, transmitted2,
 																						minref, ior, fast_fresnel,tir);
-	ns->setMode(matmodes);
+	ns->setMode(*matmodes);
 
 	for(list<paramMap_t>::iterator i=lmod.begin();i!=lmod.end();++i)
 	{
-		string shname;
 		CFLOAT	displace=0, trans=0, refle=0;
 		int color=0,  csp=0, cmir=0, ref=0,
 				specular=0, hard=0, alpha=0, emit=0;
-		string mode="mix";
 		CFLOAT colfac, varfac, def_var;
-		string texflag = "";
 		color_t texcol(1.0, 0.0, 1.0);	//blender default purple
 		color_t filtercol(1.0, 1.0, 1.0);	// texture filter color
 		CFLOAT bri=1, con=1;	// texture brightness & contrast
-		string aflag="";	// alpha flag, can contain calc_alpha, use_alpha and/or neg_alpha
 
+		string _mode="mix";
+		string _aflag="";	// alpha flag, can contain calc_alpha, use_alpha and/or neg_alpha
+		string _texflag = "";
+		string _shname;
+		const string *mode=&_mode;
+		const string *aflag=&_aflag;	// alpha flag, can contain calc_alpha, use_alpha and/or neg_alpha
+		const string *texflag = &_texflag;
+		const string *shname= &_shname;
+		
 		paramMap_t &params=*i;
 
 		params.getParam("input",shname);
-		shader_t *input=render.getShader(shname);
+		shader_t *input=render.getShader(*shname);
 		if(input==NULL)
 		{
-			cerr << "Undefined block : " << shname << endl;
+			cerr << "Undefined block : " << *shname << endl;
 			continue;
 		}
 
@@ -1090,10 +1107,10 @@ shader_t * blenderShader_t::factory(paramMap_t &bparams,std::list<paramMap_t> &l
 
 		blenderModulator_t modu(input);
 
-		if (mode=="mix") modu.mode(MIX);
-		if (mode=="mul") modu.mode(MUL);
-		if (mode=="add") modu.mode(ADD);
-		if (mode=="sub") modu.mode(SUB);
+		if (*mode=="mix") modu.mode(MIX);
+		if (*mode=="mul") modu.mode(MUL);
+		if (*mode=="add") modu.mode(ADD);
+		if (*mode=="sub") modu.mode(SUB);
 
 		modu.color(color);
 		modu.displace(displace);
@@ -1111,14 +1128,14 @@ shader_t * blenderShader_t::factory(paramMap_t &bparams,std::list<paramMap_t> &l
 		modu.setColFac(colfac);
 		modu.setDVar(def_var);
 		modu.setVarFac(varfac);
-		modu.setTexFlag(texflag);
+		modu.setTexFlag(*texflag);
 		modu.setTexCol(texcol);
 
 		modu.setFilterCol(filtercol);
 		modu.setContrast(con);
 		modu.setBrightness(bri);
 
-		modu.setAlphaFlag(aflag);
+		modu.setAlphaFlag(*aflag);
 
 		ns->addModulator(modu);
 
@@ -1131,7 +1148,7 @@ shader_t * blenderShader_t::factory(paramMap_t &bparams,std::list<paramMap_t> &l
 extern "C"
 {
 
-void registerPlugin(renderEnvironment_t &render)
+YAFRAYPLUGIN_EXPORT void registerPlugin(renderEnvironment_t &render)
 {
 	render.registerFactory("blendermapper",blenderMapperNode_t::factory);
 	render.registerFactory("blendershader",blenderShader_t::factory);
