@@ -1,3 +1,4 @@
+
 #include "imagebackg.h"
 
 // sunsky, from 'A Practical Analytic Model For DayLight" by Preetham, Shirley & Smits.
@@ -34,14 +35,15 @@ color_t imageBackground_t::operator() (const vector3d_t &dir,renderState_t &stat
 
 background_t *imageBackground_t::factory(paramMap_t &params,renderEnvironment_t &render)
 {
-  string filename;
+  string _filename;
+  const string *filename=&_filename;
   CFLOAT pwr = 1.0;	// here just a brightness multiplier, same as for lights
   params.getParam("power", pwr);
   params.getParam("filename", filename);
-  if (filename=="") {
+  if (*filename=="") {
     cerr << "(background_image) Error,  No filename given\n";
     return NULL;
   }
-  return new imageBackground_t(filename.c_str(), pwr);
+  return new imageBackground_t(filename->c_str(), pwr);
 }
 __END_YAFRAY
