@@ -17,7 +17,7 @@ __BEGIN_YAFRAY
 
 struct renderState_t;
 
-class background_t
+class YAFRAYCORE_EXPORT background_t
 {
 	public:
 		virtual color_t operator () (const vector3d_t &dir,renderState_t &state)const=0;
