@@ -29,11 +29,12 @@
 #include "color.h"
 
 __BEGIN_YAFRAY
-class colorOutput_t
+class YAFRAYCORE_EXPORT colorOutput_t
 {
 	public:
 		virtual ~colorOutput_t() {};
-		virtual void putPixel(int x, int y,const color_t &c, CFLOAT alpha=0)=0;
+		virtual bool putPixel(int x, int y,const color_t &c, 
+				CFLOAT alpha=0,PFLOAT depth=0)=0;
 		virtual void flush()=0;
 };
 
