@@ -26,50 +26,34 @@
 #include <cstdlib>
 #include<fstream>
 #include "ipc.h"
+#include "renderblock.h"
+#include "geometree.h"
 
 
 using namespace std;
-#if HAVE_PTHREAD
-
-#include<pthread.h>
-#include <semaphore.h>
-#include<map>
-
-sem_t pstop;
-//map<int,int> raylevel;
-//map<int,CFLOAT> depth;
-//map<int,CFLOAT> contribution;
-//map<int,object3d_t *> lastobject;
-//map<int,void *> lastobjectelement;
-//map<int,int> currentPass;
-//map<int,int> rayDivision;
-
-#else
-
-//int raylevel;
-//CFLOAT depth;
-//CFLOAT contribution;
-//object3d_t * lastobject;
-//void * lastobjectelement;
-//int currentPass;
-//int rayDivision;
-
-#endif
 
 __BEGIN_YAFRAY
 
 int bcount;
 int pcount;
 
+renderState_t::renderState_t() :raylevel(0),depth(0),contribution(1.0),/*lastobject(NULL)
+	,lastobjectelement(NULL),*/ skipelement(NULL),currentPass(0),rayDivision(1),traveled(0)
+	,pixelNumber(0) 
+{
+}
+
+renderState_t::~renderState_t() 
+{
+}
+
 scene_t::scene_t()
 {
 	//min_raydis=0.1;
 	//min_raydis=0.00000000000001;
 	min_raydis=MIN_RAYDIST;
+        cpus = 1;
 	self_bias=0.1;
-#ifndef HAVE_PTHREAD
-	raylevel=-1;
-#endif
 	maxraylevel=3;
 	world_resolution=1.0;
 	radio_light=NULL;
@@ -91,6 +75,26 @@ scene_t::~scene_t()
 	*/
 }
 
+void scene_t::addObject(object3d_t *obj) 
+{
+	obj_list.push_back(obj);
+}
+
+void scene_t::addLight(light_t *light) 
+{
+	light_list.push_back(light);
+}
+
+void scene_t::addFilter(filter_t *filter) 
+{
+	filter_list.push_back(filter);
+}
+
+void scene_t::publishVoidData(const std::string &key,const void *data)
+{
+	published[key]=data;
+}
+
 void scene_t::setCamera(camera_t *cam)
 {
 	render_camera=cam;
@@ -108,7 +112,8 @@ bool scene_t::isShadowed(renderState_t &state,const surfacePoint_t &sp,
 	ray.normalize();
 	point3d_t self=p+ray*self_bias;
 	p=p+ray*min_raydis;
-	object3d_t * &lasto=state.lastobject;
+	/*
+	const object3d_t * &lasto=state.lastobject;
 
 	if(lasto!=NULL)
 	{
@@ -119,20 +124,22 @@ bool scene_t::isShadowed(renderState_t &state,const surfacePoint_t &sp,
 		else
 			if(lasto->shoot(state,temp,p,ray,true,dist)) return true;
 	}
-	for(objectIterator_t ite(*BTree,p,ray,dist);!ite;ite++)
+	*/
+	//for(objectIterator_t ite(*BTree,p,ray,dist);!ite;ite++)
+	for(geomeIterator_t<object3d_t> ite(BTree,dist,p,ray);!ite;ite++)
 	{
-		if( (*ite)->castShadows() && (*ite!=lasto) )
+		if( (*ite)->castShadows()/* && (*ite!=lasto) */)
 		{
 
 			if(*ite==sp.getObject())
 			{
-				if((*ite)->shoot(state,temp,self,ray,true,dist)) {lasto=*ite;return true;}
+				if((*ite)->shoot(state,temp,self,ray,true,dist)) {/*lasto=*ite;*/return true;}
 			}
 			else
-				if((*ite)->shoot(state,temp,p,ray,true,dist)) {lasto=*ite;return true;}
+				if((*ite)->shoot(state,temp,p,ray,true,dist)) {/*lasto=*ite;*/return true;}
 		}
 	}
-	lasto=NULL;
+	//lasto=NULL;
 	return false;
 }
 
@@ -145,7 +152,8 @@ bool scene_t::isShadowed(renderState_t &state,const surfacePoint_t &sp,
 	ray.normalize();
 	point3d_t self=p+ray*self_bias;
 	p=p+ray*min_raydis;
-	object3d_t * &lasto=state.lastobject;
+	/*
+	const object3d_t * &lasto=state.lastobject;
 
 	if(lasto!=NULL)
 	{
@@ -156,19 +164,21 @@ bool scene_t::isShadowed(renderState_t &state,const surfacePoint_t &sp,
 		else
 			if(lasto->shoot(state,temp,p,ray,true)) return true;
 	}
-	for(objectIterator_t ite(*BTree,p,ray);!ite;ite++)
+	*/
+	//for(objectIterator_t ite(*BTree,p,ray);!ite;ite++)
+	for(geomeIterator_t<object3d_t> ite(BTree,numeric_limits<PFLOAT>::infinity(),p,ray);!ite;ite++)
 	{
-		if( (*ite)->castShadows() && (*ite!=lasto) )
+		if( (*ite)->castShadows()/* && (*ite!=lasto) */)
 		{
 			if(*ite==sp.getObject())
 			{
-				if((*ite)->shoot(state,temp,self,ray,true)) {lasto=*ite;return true;}
+				if((*ite)->shoot(state,temp,self,ray,true)) {/*lasto=*ite;*/return true;}
 			}
 			else
-				if((*ite)->shoot(state,temp,p,ray,true)) {lasto=*ite;return true;}
+				if((*ite)->shoot(state,temp,p,ray,true)) {/*lasto=*ite;*/return true;}
 		}
 	}
-	lasto=NULL;
+	//lasto=NULL;
 	return false;
 }
 
@@ -203,12 +213,16 @@ color_t scene_t::raytrace(renderState_t &state,const point3d_t &from,
 	point3d_t f=from+ray*min_raydis;
 	surfacePoint_t sp,temp;
 	bool found=false;
-	for(objectIterator_t ite(*BTree,f,ray);!ite;ite++)
+	//for(objectIterator_t ite(*BTree,f,ray);!ite;ite++)
+	PFLOAT limit=numeric_limits<PFLOAT>::infinity();
+	for(geomeIterator_t<object3d_t> ite(BTree,numeric_limits<PFLOAT>::infinity(),f,ray);!ite;ite++)
 	{
-		if((*ite)->shoot(state,temp,f,ray))
+		if((*ite)->shoot(state,temp,f,ray,false,limit))
 		{
 			if(temp.Z()>0.0)
 			{
+				limit=temp.Z();
+				ite.limit(limit);
 				if(!found)
 				{
 					sp=temp;
@@ -219,7 +233,7 @@ color_t scene_t::raytrace(renderState_t &state,const point3d_t &from,
 		}
 	}
 	// need to set screen position in calculated surfacepoint here for possible win texmap.
-	sp.setScreenPos(screenpos);
+	sp.setScreenPos(state.screenpos);
 	if(found && (sp.getShader()!=NULL))
 	{
 		vector3d_t eye = from-sp.P();	// also now needed in displace
@@ -278,12 +292,14 @@ color_t scene_t::raytrace_indir(renderState_t &state,const point3d_t &from,
 	point3d_t f=from+ray*min_raydis;
 	surfacePoint_t sp,temp;
 	bool found=false;
-	for(objectIterator_t ite(*BTree,f,ray);!ite;ite++)
+	//for(objectIterator_t ite(*BTree,f,ray);!ite;ite++)
+	for(geomeIterator_t<object3d_t> ite(BTree,numeric_limits<PFLOAT>::infinity(),f,ray);!ite;ite++)
 	{
 		if((*ite)->shoot(state,temp,f,ray))
 		{
 			if(temp.Z()>0.0)
 			{
+				ite.limit(temp.Z());
 				if(!found)
 				{
 					sp=temp;
@@ -296,7 +312,7 @@ color_t scene_t::raytrace_indir(renderState_t &state,const point3d_t &from,
 	if(found)
 	{
 		// need to set screen position in calculated surfacepoint here for possible win texmap.
-		sp.setScreenPos(screenpos);
+		sp.setScreenPos(state.screenpos);
 		const shader_t *sha= sp.getShader();
 		if(sha==NULL)
 		{
@@ -305,6 +321,8 @@ color_t scene_t::raytrace_indir(renderState_t &state,const point3d_t &from,
 		}
 		vector3d_t eye=from-sp.P();
 		eye.normalize();
+		PFLOAT oldtraveled=state.traveled;
+		state.traveled+=sp.Z();
 		sp.getShader()->displace(state, sp, eye, world_resolution);
 		// indirect light pass
 		color_t flights(0.0);
@@ -332,6 +350,7 @@ color_t scene_t::raytrace_indir(renderState_t &state,const point3d_t &from,
 		l_depth=(from-sp.P()).length();
 		// add simple fog if enabled
 		fog_addToCol(l_depth, flights);
+		state.traveled=oldtraveled;
 		return flights;
 	}
 	l_raylevel--;
@@ -360,7 +379,7 @@ void scene_t::postSetupLights()
 			++ite)
 		(*ite)->postInit(*this);
 }
-
+/*
 bool scene_t::checkSampling()
 {
 	bool need = false;
@@ -436,7 +455,7 @@ bool scene_t::checkSampling()
 
 
 bool scene_t::doOnePass(renderState_t &state,
-		int thread,vector<color_t> &line,
+                vector<color_t> &line,
 		vector<CFLOAT> &dep,
 		vector<CFLOAT> &alpha,int numline,int pass)
 {
@@ -458,6 +477,7 @@ bool scene_t::doOnePass(renderState_t &state,
 			dep[j] = -1;
 			contri = 1.0;
 			globalpass=0;
+			state.pixelNumber=j+numline*colorBuffer.resx();
 			if((screenpos.x<scxmin) || (screenpos.x>scxmax) || (screenpos.y<scymin) || (screenpos.y>scymax))
 				fcol=color_t(0,0,0);
 			else
@@ -475,6 +495,7 @@ bool scene_t::doOnePass(renderState_t &state,
 			if (totsamdiv!=0) totsamdiv = 1.0/totsamdiv;
 			unsigned int cursam;
 			CFLOAT tot_alpha = 0;	// alpha AA
+			state.pixelNumber=j+numline*colorBuffer.resx();
 			for (int ms=0;ms<AA_minsamples;ms++) {
 			  globalpass = cursam = (pass-1)*AA_minsamples + ms;
 			  fx = AA_pixelwidth*(fast_vdC(cursam) - 0.5);
@@ -501,7 +522,7 @@ bool scene_t::doOnePass(renderState_t &state,
 
 
 bool scene_t::doAllPasses(renderState_t &state,
-		int thread,vector<color_t> &line,
+                vector<color_t> &line,
 		vector<CFLOAT> &dep,
 		vector<CFLOAT> &alpha,int numline)
 {
@@ -515,6 +536,7 @@ bool scene_t::doAllPasses(renderState_t &state,
 		color_t sum(0.0), avg(0.0), var(0.0);
 		int i;
 		PFLOAT fx, fy, sc;
+		state.pixelNumber=j+numline*colorBuffer.resx();
 		for(i=0;i<AA_onepass_max;++i)
 		{
 			// have to use Halton here
@@ -550,7 +572,7 @@ bool scene_t::doAllPasses(renderState_t &state,
 }
 
 
-void scene_t::renderPart(colorOutput_t &out, int curpass, int cpus, int off)
+void scene_t::renderPart(colorOutput_t &out, int curpass, int off)
 {
 	vector3d_t ray;
 	color_t color;
@@ -561,9 +583,6 @@ void scene_t::renderPart(colorOutput_t &out, int curpass, int cpus, int off)
 	resx=render_camera->resX();
 	resy=render_camera->resY();
 	state.raylevel=-1;
-#if HAVE_PTHREAD
-	sem_wait(&pstop);
-#endif
 	vector<CFLOAT> dep(resx);
 	vector<CFLOAT> alpha(resx);
 	vector<color_t> line(resx);
@@ -579,13 +598,15 @@ void scene_t::renderPart(colorOutput_t &out, int curpass, int cpus, int off)
 			colorBuffer(j,i)>>line[j];
 			// need to fill in alpha scan buffer too,
 			// since AA is done on the alpha channel as well
-			if (curpass>0) alpha[j]=ABuffer(j,i);
+                        if (curpass>0)
+                            alpha[j]=ABuffer(j,i);
 		}
 		doPass(state,0, line, dep, alpha, i, curpass);
 		for(int j=0;j<resx;++j)
 		{
 			colorBuffer(j,i)<<line[j];
-			if (curpass==0) ZBuffer(j,i)=dep[j];
+                        if (curpass==0)
+                            ZBuffer(j,i)=dep[j];
 			// must save alpha for AA
 			ABuffer(j,i) = alpha[j];
 			out.putPixel(j, i,line[j],alpha[j]);
@@ -598,301 +619,76 @@ void scene_t::renderPart(colorOutput_t &out, int curpass, int cpus, int off)
 		count++;
 	}
 }
+*/
 
-#ifdef FORCE_FORKS
 
-void scene_t::render(colorOutput_t &out, int cpus)
+// Support for single-process rendering
+void scene_t::render(colorOutput_t &out)
 {
 	int resx,resy;
-	int steps;
-	int CS=1;
-	int child=0;
-	int iFork=0;
 	resx=render_camera->resX();
 	resy=render_camera->resY();
-	vector3d_t ray;
-	color_t color;
+	blockSpliter_t spliter(resx,resy,64);
 
-	fprintf(stderr,"Building the bounding tree ... ");
-	fflush(stderr);
-	
-	BTree=new boundTree_t (obj_list);
-	
-	cout<<"OK\n";
-	
+	renderArea_t area;
+
+	cout<<"Building bounding tree ... ";cout.flush();
+	//BTree=new boundTree_t (obj_list);
+	BTree=buildObjectTree (obj_list);
+	cout<<"OK"<<endl;
+
+	cout<<"Light setup ..."<<endl;
 	setupLights();
 
-	colorBuffer.set(resx,resy);
-	ZBuffer.set(resx,resy);
-	ABuffer.set(resx,resy);
-	oversample.set(resx,resy);
-
-	for(int i=0;i<resy;++i)
-		for(int j=0;j<resx;++j)
-			oversample(j,i)=1;
-
-
-	steps=resy/(int)(resy/STEPS);
-	steps=cpus*((int)steps/cpus);
-
-	cout << "0%";
-	for(int i=0;i<((steps+1)/2)-3;++i) cout<<" ";
-	cout << "50%";
-	for(int i=0;i<((steps+1)/2)-3;++i) cout<<" ";
-	cout << "100%\n";
-
-	pid_t waitFork[cpus];
-	vector<pair<int,int> > pipeVectorStoF(cpus);
-	vector<pair<int,int> > pipeVectorFtoS(cpus);
-
-	cout<<"\r[";
-	for(int i=0;i<steps;++i) cout<<".";
-
-	cout<<"] first render pass\r[";
-	cout.flush();
-
-	lastobject=NULL;
-	lastobjectelement=NULL;
-	currentPass=0;
-	rayDivision=1;
-
-	for(iFork=0;iFork<cpus;++iFork)
+	cout<<endl;
+	while(repeatFirst)
 	{
-			int temp[2];
-			pipe(temp);
-			pipeVectorStoF[iFork].first=temp[0];
-			pipeVectorStoF[iFork].second=temp[1];
-			pipe(temp);
-			pipeVectorFtoS[iFork].first=temp[0];
-			pipeVectorFtoS[iFork].second=temp[1];
-
-			if((waitFork[iFork]=fork())==0) //Son send
-			{
-				child=1;
-				close(pipeVectorStoF[iFork].first);
-				close(pipeVectorFtoS[iFork].second);
-				renderPart(out,0,cpus,iFork);
-				sendColor(colorBuffer,pipeVectorStoF[iFork].second,resx,resy,cpus,iFork);
-				sendFloat(ZBuffer,pipeVectorStoF[iFork].second,resx,resy,cpus,iFork);
-				sendFloat(ABuffer,pipeVectorStoF[iFork].second,resx,resy,cpus,iFork);
-				break;
-			}
-			else
-			{
-				close(pipeVectorStoF[iFork].second);
-				close(pipeVectorFtoS[iFork].first);
-			}
-	}
-
-	if(!child)
-	{
-		mixColor(colorBuffer,resx,resy,cpus,pipeVectorStoF);
-		mixFloat(ZBuffer,resx,resy,cpus,pipeVectorStoF);
-		mixFloat(ABuffer,resx,resy,cpus,pipeVectorStoF);
-
-		if ((!checkSampling()) || (AA_onepass_max)) CS = 0;
-	}
-
-	for(int pass=1;pass<(AA_passes+1) && CS;++pass)
-	{
-		lastobject=NULL;
-		lastobjectelement=NULL;
-		currentPass=0;
-		rayDivision=1;
-
-		if(!child) //Father send && receive
-		{
-			cout<<"\r[";
-			for(int i=0;i<steps;++i) cout<<".";
-
-			cout << "] AA pass " << pass << "\r";
-			cout << "[";
-			cout.flush();
-
-			for(int iFork=0;iFork<cpus;iFork++)
-				writePipe(pipeVectorFtoS[iFork].second,&CS,sizeof(int)); //CheckSampling
-
-			sendNOversample(oversample,pipeVectorFtoS,resx,resy,cpus);
-
-			mixColor(colorBuffer,resx,resy,cpus,pipeVectorStoF);
-
-			cout<<"]                  ";
-
-			if(!checkSampling()) CS = 0;
-
-		}
-		else //Son receive
-		{
-			readPipe(pipeVectorFtoS[iFork].first,&CS,sizeof(int));
-			if(!CS) //No checkSampling die
-				break;
-
-			receiveOversample(oversample,resx,resy,pipeVectorFtoS[iFork].first);
-			renderPart(out, pass, cpus, iFork);
-			sendColor(colorBuffer,pipeVectorStoF[iFork].second,resx,resy,cpus,iFork);
-		}
-	}
-
-	//End
-
-	if(child)
-	{
-		close(pipeVectorStoF[iFork].second);
-		close(pipeVectorFtoS[iFork].first);
-		delete BTree;
-		BTree=NULL;
-		exit(0);
-	}
-
-	if (!CS) //No checkSampling kill Sons
-	{
-		for(iFork=0;iFork<cpus;iFork++)
-			writePipe(pipeVectorFtoS[iFork].second,&CS,sizeof(int));
-	}
-	
-	for(int iFork=0;iFork<cpus;++iFork)
-	{
-		waitpid(waitFork[iFork],(int *)0,(int)0);
-	}
-
-	for(int i=0;i<cpus;i++)
-	{
-		close(pipeVectorStoF[i].first);
-		close(pipeVectorFtoS[i].second);
-	}
-
-	cout << "\nForks finished.\n";
-
-	for(list<filter_t *>::iterator ite=filter_list.begin();ite!=filter_list.end();
-			ite++)
-		(*ite)->apply(colorBuffer,ZBuffer,ABuffer);
-
-	for(int i=0;i<resy;++i)
-		for(int j=0;j<resx;++j)
-		{
-			colorBuffer(j, i) >> color;
-			out.putPixel(j,i,color, ABuffer(j, i));
-		}
-}
-
-#elif HAVE_PTHREAD
-
-struct render_part_t
-{
-	scene_t *scene;
-	colorOutput_t *out;
-	int cpus, offset, curpass;
-};
-
-void * render_thread(void *param)
-{
-	render_part_t *part=(render_part_t *)param;
-
-	part->scene->renderPart(*(part->out), part->curpass, part->cpus, part->offset);
-	pthread_exit(0);
-}
-
-void scene_t::render(colorOutput_t &out, int cpus)
-{
-	int resx,resy;
-	int steps;
-	resx=render_camera->resX();
-	resy=render_camera->resY();
-	vector<pthread_t> tid(cpus);
-	pthread_attr_t attr;
-	vector<render_part_t> part(cpus);
-
-
-	fprintf(stderr,"Building the bounding tree ... ");
-	fflush(stderr);
-	BTree=new boundTree_t (obj_list);
-	cout<<"OK\n";
-	setupLights();
-
-	cout<<"Launching "<<cpus<<" threads for rendering ...\n";
-	cout<<"Rendering ...\n";
-
-	colorBuffer.set(resx,resy);
-	ZBuffer.set(resx,resy);
-	ABuffer.set(resx,resy);
-	oversample.set(resx,resy);
-
-	for(int i=0;i<resy;++i)
-		for(int j=0;j<resx;++j)
-			oversample(j,i)=1;
-
-
-	steps=resy/(int)(resy/STEPS);
-	steps=cpus*((int)steps/cpus);
-
-	cout << "0%";
-	for(int i=0;i<((steps+1)/2)-3;++i) cout<<" ";
-	cout << "50%";
-	for(int i=0;i<((steps+1)/2)-3;++i) cout<<" ";
-	cout << "100%\n";
-
-	int numpass;
-	if (AA_onepass_max) numpass=1; else numpass=AA_passes+1;
-	for (int pass=0;pass<numpass;++pass)
-	{
-		cout<<"\r[";
-		for(int i=0;i<steps;++i) cout<<".";
-		if (pass==0)
-			cout << "] first render pass\r";
-		else
-			cout << "] AA pass " << pass << "\r";
-		cout<<"[";
+		cout<<"\rFake   pass: [";
 		cout.flush();
-		sem_init(&pstop, 0, 0);
-		pthread_attr_init(&attr);
-		pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_JOINABLE);
-		for(int i=0;i<cpus;++i)
+		repeatFirst=false;
+		blockSpliter_t fakespliter(resx,resy,64);
+		int finished=0;
+		
+		while(!fakespliter.empty())
 		{
-			part[i].scene=this;
-			part[i].out=&out;
-			part[i].curpass=pass;
-			part[i].cpus=cpus;
-			part[i].offset=i;
-			pthread_create(&tid[i], &attr, render_thread,(void *)&part[i]);
+			if((finished>0) && !(finished%10)) {cout<<"#";cout.flush();}
+			fakespliter.getArea(area);
+			fakeRender(area);
+			if(!area.out(out))
+			{
+				cout<<"Aborted"<<endl;
+				delete BTree;
+				BTree=NULL;
+				return;
+			}
+			finished++;
 		}
-		pthread_attr_destroy(&attr);
-		for(int i=0;i<cpus;++i)
-			sem_post(&pstop);
-		for(int i=0;i<cpus;++i)
-		    pthread_join(tid[i], NULL);
-		cout<<"]                  ";
-		sem_destroy(&pstop);
-		if(repeatFirst && (pass==0))
-		{
-			repeatFirst=false;
-			--pass;
-			cout<<"\nLight needs post init ...";cout.flush();
-			postSetupLights();
-			cout<<"OK, repeating first pass\n";
-		}
-		else if (!checkSampling()) break;
+		cout<<"#]"<<endl;
+		postSetupLights();
 	}
-	cout<<"\nThreads finished\n";
+	cout<<endl;
+
+	cout<<"\rRender pass: [";
+	cout.flush();
+	int finished=0;
+	while(!spliter.empty())
+	{
+		if((finished>0) && !(finished%10)) {cout<<"#";cout.flush();}
+		spliter.getArea(area);
+		render(area);
+		if(!area.out(out))
+		{
+			cout<<"Aborted"<<endl;
+			delete BTree;
+			BTree=NULL;
+			return;
+		}
+		finished++;
+	}
+	cout<<"#]"<<endl;
 	delete BTree;
 	BTree=NULL;
-
-	for(list<filter_t *>::iterator ite=filter_list.begin();ite!=filter_list.end();
-			ite++)
-		(*ite)->apply(colorBuffer,ZBuffer,ABuffer);
-	
-	color_t color;
-	for(int i=0;i<resy;++i)
-		for(int j=0;j<resx;++j)
-		{
-			colorBuffer(j, i) >> color;
-			out.putPixel(j, i, color, ABuffer(j, i));
-		}
-}
-
-#else
-
-void scene_t::render(colorOutput_t &out, int cpus)
-{
+	/*
 	int resx,resy;
 	int steps;
 	resx=render_camera->resX();
@@ -900,18 +696,15 @@ void scene_t::render(colorOutput_t &out, int cpus)
 	vector3d_t ray;
 	color_t color;
 
+
 	fprintf(stderr,"Building the bounding tree ... ");
 	fflush(stderr);
 	BTree=new boundTree_t (obj_list);
 	cout<<"OK\n";
 	setupLights();
 
+        cout<<"Rendering in monolithic mode ...\n";
 
-	if(raylevel!=-1)
-	{
-		cout<<"Error, bad status\n";
-		exit(1);
-	}
 
 	colorBuffer.set(resx,resy);
 	ZBuffer.set(resx,resy);
@@ -924,6 +717,8 @@ void scene_t::render(colorOutput_t &out, int cpus)
 
 
 	steps=resy/(int)(resy/STEPS);
+
+
 	cout << "0%";
 	for(int i=0;i<((steps+1)/2)-3;++i) cout<<" ";
 	cout << "50%";
@@ -933,6 +728,7 @@ void scene_t::render(colorOutput_t &out, int cpus)
 	bool first=true;
 	int numpass;
 	if (AA_onepass_max) numpass=1; else numpass=AA_passes+1;
+        cout << "Rendering in "<<numpass<<" passes\n";
 	for(int pass=0;pass<numpass;++pass)
 	{
 		cout<<"\r[";
@@ -943,11 +739,7 @@ void scene_t::render(colorOutput_t &out, int cpus)
 			cout << "] AA pass " << pass << "\r";
 		cout << "[";
 		cout.flush();
-		lastobject=NULL;
-		lastobjectelement=NULL;
-		currentPass=0;
-		rayDivision=1;
-		renderPart(out, pass, 1,0);
+                renderPart(out, pass,0);
 		cout << "]                  ";
 
 		if(repeatFirst && first)
@@ -974,9 +766,9 @@ void scene_t::render(colorOutput_t &out, int cpus)
 			colorBuffer(j,i) >> color;
 			out.putPixel(j, i, color, ABuffer(j, i));
 		}
+	*/
 }
 
-#endif
 
 bool scene_t::firstHit(renderState_t &state,surfacePoint_t &sp,const point3d_t &from,
 											const vector3d_t &ray,bool shadow)const
@@ -984,13 +776,15 @@ bool scene_t::firstHit(renderState_t &state,surfacePoint_t &sp,const point3d_t &
 	surfacePoint_t temp;
 	bool found=false;
 	point3d_t f=from+ray*min_raydis;
-	for(objectIterator_t ite(*BTree,f,ray);!ite;ite++)
+	//for(objectIterator_t ite(*BTree,f,ray);!ite;ite++)
+	for(geomeIterator_t<object3d_t> ite(BTree,numeric_limits<PFLOAT>::infinity(),f,ray);!ite;ite++)
 	{
 		if(shadow && !(*ite)->castShadows()) continue;
 		if((*ite)->shoot(state,temp,f,ray))
 		{
 			if(temp.Z()>0.0)
 			{
+				ite.limit(temp.Z());
 				if(!found)
 				{
 					sp=temp;
@@ -1003,9 +797,115 @@ bool scene_t::firstHit(renderState_t &state,surfacePoint_t &sp,const point3d_t &
 	if(found && !shadow && (sp.getShader()!=NULL)) {
 		vector3d_t eye = from-sp.P();		// is needed in displace
 		eye.normalize();
+		PFLOAT oldtraveled=state.traveled;
+		state.traveled+=sp.Z();
 		sp.getShader()->displace(state, sp, eye, world_resolution);
+		state.traveled=oldtraveled;
 	}
 	return found;
 }
 
+void scene_t::render(renderArea_t &area)const
+{
+	renderState_t state;
+	CFLOAT &contri=state.contribution;
+	CFLOAT &pdep=state.depth;
+	int &globalpass=state.currentPass;
+	int resx=render_camera->resX();
+	int resy=render_camera->resY();
+	// now includes alpha channel as well, no separate calculation
+	colorA_t fcol;
+
+	//First pass
+	for(int i=area.Y;i<(area.Y+area.H);++i)
+		for(int j=area.X;j<(area.X+area.W);++j)
+		{
+			state.raylevel=-1;
+			state.screenpos.set(2.0*((PFLOAT)j/(PFLOAT)resx)-1.0, 
+					1.0-2.0*((PFLOAT)i/(PFLOAT)resy), 0);
+			vector3d_t ray=render_camera->shootRay(j, i);
+			contri = 1.0;
+			globalpass=0;
+			state.pixelNumber=j+i*resx;
+			if ((state.screenpos.x>=scxmin) && (state.screenpos.x<scxmax) && 
+					(state.screenpos.y>=scymin) && (state.screenpos.y<scymax))
+			{
+				fcol = raytrace(state,render_camera->position(),ray);
+				fcol.expgam_Adjust(exposure, gamma_R);
+				if (pdep>=0) fcol.setAlpha(1.0); else fcol.setAlpha(0.0);
+				area.imagePixel(j,i) = fcol;
+				area.depthPixel(j,i) = pdep;
+			}
+		}
+
+	PFLOAT fx, fy, totsamdiv = AA_minsamples*AA_passes;
+	if (totsamdiv!=0) totsamdiv = 1.0/totsamdiv;
+	for(int pass=0;pass<AA_passes;pass++)
+	{
+		area.checkResample(AA_threshold);
+		for(int i=area.Y;i<(area.Y+area.H);++i)
+			for(int j=area.X;j<(area.X+area.W);++j)
+			{
+				if(!area.resamplePixel(j,i)) continue;
+				colorA_t totcol(0.0);
+				unsigned int cursam;
+				state.pixelNumber=j+i*resx;
+				
+				state.raylevel=-1;
+
+				for (int ms=0;ms<AA_minsamples;ms++) 
+				{
+					globalpass = cursam = pass*AA_minsamples + ms;
+					state.raylevel=-1;
+					fx = AA_pixelwidth*(fast_vdC(cursam) - 0.5);
+					fy = AA_pixelwidth*((cursam+0.5)*totsamdiv - 0.5);
+					state.screenpos.set(2.0*(((PFLOAT)j+fx)/(PFLOAT)resx)-1.0, 
+							1.0-2.0*(((PFLOAT)i+fy)/(PFLOAT)resy), 0);
+					vector3d_t ray = render_camera->shootRay((PFLOAT)j+fx, (PFLOAT)i+fy);
+					if ((state.screenpos.x>=scxmin) && (state.screenpos.x<scxmax) &&
+							(state.screenpos.y>=scymin) && (state.screenpos.y<scymax))
+					{
+						fcol = raytrace(state,render_camera->position(), ray);
+						fcol.expgam_Adjust(exposure, gamma_R);
+						if (pdep>=0) fcol.setAlpha(fcol.getAlpha()+1.0);
+						totcol += fcol;
+					}
+					
+				}
+				CFLOAT mf = (CFLOAT)(pass*AA_minsamples+1);
+				area.imagePixel(j,i) = (mf*area.imagePixel(j,i) + totcol) / (mf+(CFLOAT)AA_minsamples);
+			}
+	}
+}
+
+void scene_t::fakeRender(renderArea_t &area)const
+{
+	renderState_t state;
+	CFLOAT &contri=state.contribution;
+	int &globalpass=state.currentPass;
+	int resx=render_camera->resX();
+	int resy=render_camera->resY();
+
+	//First pass
+	for(int i=area.Y;i<(area.Y+area.H);++i)
+		for(int j=area.X;j<(area.X+area.W);++j)
+		{
+			state.raylevel=-1;
+			state.screenpos.set(2.0*((PFLOAT)j/(PFLOAT)resx)-1.0, 
+					1.0-2.0*((PFLOAT)i/(PFLOAT)resy), 0);
+			vector3d_t ray=render_camera->shootRay(j, i);
+			contri = 1.0;
+			globalpass=0;
+			state.pixelNumber=j+i*resx;
+			if(!((state.screenpos.x<scxmin) || (state.screenpos.x>scxmax) || 
+					(state.screenpos.y<scymin) || (state.screenpos.y>scymax)))
+				area.imagePixel(j,i)=raytrace(state,render_camera->position(),ray);
+		}
+}
+
+scene_t *scene_t::factory()
+{
+	return new scene_t();
+}
+
 __END_YAFRAY
